package com.chinacreator.access.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chinacreator.access.exception.BusinessException;
import com.chinacreator.access.service.*;
import com.chinacreator.access.util.IpAddrUtil;
import com.chinacreator.lock.redisson.annotation.DistributedLock;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.util.Util;
import redis.clients.jedis.JedisCluster;

import java.text.SimpleDateFormat;
import java.util.Date;


/**
 * 省分(北分）签约后向定向产品白名单用户领取
 */
@RestController
@RequestMapping("/allowlistdarw/")
public class AllowlistDrawController {

    private static final Logger LOGGER = LoggerFactory.getLogger(AllowlistDrawController.class);

    @Autowired
    private AllowlistDrawService allowlistDrawService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private JedisCluster jedisCluster;

    @DistributedLock()
    @RequestMapping(value = "/order", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String order(HttpServletRequest request, HttpServletResponse response) throws Exception {
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        JSONObject data = TransUtil.reqToAliJSON(request);
        String orderchannel = Util._N_to_10(data.getString("orderchannel"), 32);
        data.put("orderchannel", orderchannel);
        String remoteIp = IpAddrUtil.getIpAddr(request);

        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            String redisKey = sdf.format(new Date()) + remoteIp;
            String times = jedisCluster.get(redisKey);

            if (times == null) {
                jedisCluster.setex(redisKey, 60 * 60 * 24,"0");
            } else if (Integer.valueOf(times) <= 20) {
                jedisCluster.setex(redisKey, 60 * 60 * 24, String.valueOf(Integer.valueOf(times) + 1));
            } else {
                jedisCluster.setex(redisKey, 60 * 60 * 24, String.valueOf(Integer.valueOf(times) + 1));
                result.put("resultCode", "8601");
                result.put("errorInfo", "请勿频繁操作!");
                return result.toString();
            }

//            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
//            String usermob = usermobService.decodeUserId(userid, cpid);
            String ticket = data.getString("ticket");

            if (StringUtils.isEmpty(ticket)) {
                result.put("resultCode", "8602");
                result.put("errorInfo", "缺少必要参数ticket!");
                return result.toString();
            }

            data.put("ticket", ticket);
            data.put("cpid", cpid);
            data.put("productid", productid);

            LOGGER.info("白名单签约产品领取，入参={}", data);

            result = allowlistDrawService.order(data);

        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", "");

            LOGGER.info("白名单签约产品领取，入参={}，返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return result.toString();
    }
}
//    private static void inowhite(String channel,String key){
//        String url="http://100.0.4.69:3344/activity/ino/receive";
//        Map<String,Object> inmap=new HashMap<>();
//        inmap.put("channel",channel);
//        inmap.put("usermob","18612714515");
//        inmap.put("dicode","2022051001");
//        inmap.put("orderchannel","99");
//        inmap.put("sign",SignatureUtil.signatureStr(inmap,key));
//        try {
//            System.out.println(HttpClientUtil.httpClientOfPost(url, inmap));
//        }catch (Exception e){
//            e.printStackTrace();
//        }
//    }