package com.chinacreator.access.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chinacreator.access.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.support.AbstractBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.IpAddrUtil;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.util.Util;
import org.springframework.web.context.support.WebApplicationContextUtils;

/**
 * 后向流量
 */
@RestController
@RequestMapping("/backward/")
public class BackController {

    private static final Logger LOGGER = LoggerFactory.getLogger(BackController.class);

    @Autowired
    private WomusicService womusicService;

    @Autowired
    private StackpkgService stackpkgService;

    @Autowired
    private UsermobService usermobService;
    
    @Autowired
    private BackService backService;

    @Autowired
    private SliderCaptchaService sliderCaptchaService;

    @Autowired
    private UserService userService;

    @Autowired
    private BackRbtService backRbtService;

    /**
     * 查询stockcode是否使用过
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/queryStock", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject queryStock(HttpServletRequest request, HttpServletResponse response) throws Exception {
        JSONObject data = TransUtil.reqToAliJSON(request);
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");
        String operip = IpAddrUtil.getIpAddr(request);

        data.put("cpid", cpid);
        data.put("productid", productid);
        data.put("operip", operip);
        data.put("stockcode", data.getString("stockcode"));

        LOGGER.info("沃音乐查询库存，入参={}", data);

        return womusicService.queryStock(data);
    }

    /**
     * 后向产品产品订购
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/order", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject order(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        JSONObject data = null;
        try {
            data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            String operip = IpAddrUtil.getIpAddr(request);
            String usermob = usermobService.decodeUserId(userid, cpid);
            String requestIp = IpAddrUtil.getIpAddr(request);

            data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("operip", operip);
            data.put("orderchannel", orderchannel);

            LOGGER.info("后向产品订购，入参={}", data);

            switch (productid) {
                case "2020011301":
                    // 今日头条后向流量包（沃音乐渠道）
                    data.put("stockcode", data.getString("stockcode"));
                    result = womusicService.order(data);

                    break;
                case "2020011302":
                    // 今日头条后向流量包（叠加包渠道）
                    data.put("attach", "repeat_cpid");
                    result = stackpkgService.order(data, requestIp);

                    break;
                case "2020090401":
                    // 抖音权益包叠加包
                    data.put("attach", "repeat_15_cpid");
                    result = stackpkgService.order(data, requestIp);

                    break;
                case "2020050101":
                	result = backService.order(data, requestIp, "season_pack_cpid", "今日头条定向流量季包");

                    break;
                case "2021011302":
                    // 抖音视频彩铃后向流量包
                    result = backRbtService.order(data, requestIp);

                    break;
                default:
                    result.put("resultCode", "8001");
                    result.put("errorInfo", "参数错误");
                    break;
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        LOGGER.info("后向产品订购，入参={}，返回={}", data, result);
        return result;
    }

    @RequestMapping(value = "/cancel", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject cancel(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        JSONObject data = null;
        try {
            data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            String operip = IpAddrUtil.getIpAddr(request);
            String usermob = usermobService.decodeUserId(userid, cpid);
            String requestIp = IpAddrUtil.getIpAddr(request);

            data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("operip", operip);
            data.put("orderchannel", orderchannel);

            LOGGER.info("后向产品退订，入参={}", data);

            switch (productid) {
                case "2021011302":
                    // 抖音视频彩铃后向流量包
                    result = backRbtService.cancel(data, requestIp);

                    break;
                default:
                    result.put("resultCode", "8001");
                    result.put("errorInfo", "参数错误");
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        LOGGER.info("后向产品退订，入参={}，返回={}", data, result);
        return result;
    }

    /**
     * 南研签约与支付接口
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/nanyansign", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject nanyanSign(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        try {
            JSONObject data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String operip = IpAddrUtil.getIpAddr(request);
            String usermob = usermobService.decodeUserId(userid, cpid);
            String requestIp = IpAddrUtil.getIpAddr(request);

            data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("operip", operip);

            LOGGER.info("南研支付的后向产品订购，入参={}", data);

            result = backService.nanyanSign(data, requestIp, "toutiao_2020110401");

        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    /**
     * 南研支付取消签约
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/nanyanunsign", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject nanyanUnsign(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        try {
            JSONObject data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String operip = IpAddrUtil.getIpAddr(request);
            String usermob = usermobService.decodeUserId(userid, cpid);
            String requestIp = IpAddrUtil.getIpAddr(request);

            data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("operip", operip);

            LOGGER.info("后向产品订购，入参={}", data);

            result = backService.nanyanUnsign(data, requestIp, "toutiao_2020110401");

        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    /**
     * 沃音乐退款
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/refund", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject refund(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();

        try {
            JSONObject data = TransUtil.reqToAliJSON(request);
            //String userid = data.getString("userid");
            //String cpid = Util.fogdec(data.getString("fog"), "cpid");
            //String productid = Util.fogdec(data.getString("fog"), "productid");
            String operip = IpAddrUtil.getIpAddr(request);
            //String usermob = usermobService.decodeUserId(userid, cpid);

            /* data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("operip", operip);
            data.put("stockcode", data.getString("stockcode")); */

            boolean verifyResult = sliderCaptchaService.verifyCaptChaCode(data.getString("captchaCode"));

            if (!verifyResult) {
                result.put("resultCode", "8002");
                result.put("errorInfo", "参数错误");
                return result;
            }

            boolean checkUser = userService.checkUser(data.getString("username"), data.getString("password"));
            if (!checkUser) {
                result.put("resultCode", "8003");
                result.put("errorInfo", "登录账号或密码错误");
                return result;
            }

            return womusicService.refund(data.getString("stockcode"), operip);

        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", "");

            e.printStackTrace();
        } finally {

        }

        return result;
    }

    /**
     * 订购关系查询
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/stackpkgqueryorder", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject stackpkgQueryOrder(HttpServletRequest request, HttpServletResponse response) throws Exception {
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        try {
            JSONObject data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String usermob = usermobService.decodeUserId(userid, cpid);
            String tradeno = data.getString("tradeno");

            result = stackpkgService.queryOrder(usermob, tradeno);
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", "");

            e.printStackTrace();
        }

        return result;
    }

    /**
     * 微信支付结果查询
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping(value = {"/stackpkgquerypay", "/wxpayquery"}, method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject wxPayQuery(HttpServletRequest request, HttpServletResponse response) throws Exception {
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        try {
            JSONObject data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String usermob = "";
            if (!StringUtils.isEmpty(userid)) {
                usermob = usermobService.decodeUserId(userid, cpid);
            }

            LOGGER.info("微信支付结果查询，入参={}，userid={}，usermob={}", data, userid, usermob);

            String tradeno = data.getString("tradeno");
            String tradedate = data.getString("tradedate");

            result = stackpkgService.queryPay(tradeno, tradedate, usermob);
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", "");

            e.printStackTrace();
        }

        return result;
    }

    /**
     * 南研支付结果查询
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @RequestMapping(value = {"/queryPay"}, method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject queryPay(HttpServletRequest request, HttpServletResponse response) throws Exception {
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        try {
            JSONObject data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String usermob = "";
            if (!StringUtils.isEmpty(userid)) {
                usermob = usermobService.decodeUserId(userid, cpid);
            }
            String operip = IpAddrUtil.getIpAddr(request);

            LOGGER.info("南研支付结果查询，入参={}，userid={}，usermob={}", data, userid, usermob);

            result = backRbtService.queryPay(data, operip);
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", "");

            e.printStackTrace();
        }

        return result;
    }
}
