package com.chinacreator.access.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.service.ChannelInfoService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.restrain.ReqEnum;
import com.chinacreator.access.service.ChoiceOrderService;
import com.chinacreator.access.service.UsermobService;
import com.chinacreator.access.util.IpAddrUtil;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.util.Util;

/**
 * 多选一产品的订购
 * 
 * @author jiangtao
 *
 */
@RestController
@RequestMapping(value = "/choiceorder")
public class ChoiceOrderController {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(ChoiceOrderController.class);

	@Autowired
	private ChoiceOrderService choiceOrderService;

	@Autowired
	private UsermobService usermobService;

	@Autowired
	private ChannelInfoService channelInfoService;

	/**
	 * 订购
	 */
	@RequestMapping(value = "/order", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
	public String order(HttpServletRequest request, HttpServletResponse response) {
		try {
			JSONObject data = TransUtil.reqToAliJSON(request);
			String userid = data.getString("userid");
			String cpid = Util.fogdec(data.getString("fog"), "cpid");
			String productid = Util.fogdec(data.getString("fog"), "productid");
			String orderchannel = Util._N_to_10(data.getString("sou"), 32);
			ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
			String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();

			data.put("usermob", usermobService.decodeUserId(userid, cpid));
			data.put("cpid", cpid);
			data.put("productid", productid);
			data.put("requestIp", IpAddrUtil.getIpAddr(request));
			data.put("orderchannel", orderchannel);
			data.put("smschannel", smschannel);

			LOGGER.info("多选一产品订购，入参={}", data);

			JSONObject jsonObject = choiceOrderService.order(data);
			if ("6631".equals(jsonObject.getString("resultCode"))) {
				jsonObject.put("errorInfo", "用户已欠费，请缴费后重新发起！");
			}
			return jsonObject.toJSONString();

		} catch (Exception e) {
			JSONObject data = Util.reqAliJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
			LOGGER.error("多选一产品订购失败，入参={}, 返回={}", data, e.getMessage());
			e.printStackTrace();

			return data.toString();
		}
	}

	/**
	 * 退订
	 */
	@RequestMapping(value = "/cancel", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
	public String cancel(HttpServletRequest request, HttpServletResponse response) {
		JSONObject result = null;
		try {
			JSONObject data = TransUtil.reqToAliJSON(request);
			String userid = data.getString("userid");
			String cpid = Util.fogdec(data.getString("fog"), "cpid");
			String productid = Util.fogdec(data.getString("fog"), "productid");
			String orderchannel = Util._N_to_10(data.getString("sou"), 32);
			ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
			String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();

			data.put("usermob", usermobService.decodeUserId(userid, cpid));
			data.put("cpid", cpid);
			data.put("productid", productid);
			data.put("requestIp", IpAddrUtil.getIpAddr(request));
			data.put("smschannel", smschannel);

			LOGGER.info("多选一产品退订，入参={}", data);

			result = choiceOrderService.cancel(data);
		} catch (Exception e) {
			JSONObject data = Util.reqAliJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
			LOGGER.error("多选一产品退订失败，入参={}，返回={}", data, e.getMessage());
			e.printStackTrace();
			return data.toString();
		}

		return result.toString();
	}

}
