package com.chinacreator.access.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.service.ChannelInfoService;
import com.chinacreator.lock.redisson.annotation.DistributedLock;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.service.DouyinCoinService;
import com.chinacreator.access.service.UsermobService;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.Util;

/**
 * 虚拟货币权益
 */
@RestController
@RequestMapping("/coin")
public class CoinController {

    private static final Logger LOGGER = LoggerFactory.getLogger(CoinController.class);

    @Autowired
    private DouyinCoinService douyinCoinService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private ChannelInfoService channelInfoService;

    @Autowired
    public static HttpSendService send = new HttpSendServicImp();
    
    @Value("${sendUrl}")
    private String sendUrl;

    /**
     * 查询领取情况
     */
    @RequestMapping(value = "/query", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String query(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(request);

        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");
        String uid = data.getString("uid");
        String userid = data.getString("userid");
        String orderchannel = Util._N_to_10(data.getString("sou"), 32);

        JSONObject result = new JSONObject();

        try {
            data.put("uid", uid);
            data.put("usermob", usermobService.decodeUserId(userid, cpid));
            data.put("cpid", cpid);
            data.put("productid", productid);

            String returnStr = douyinCoinService.query(data).toString();
            LOGGER.info("查询抖音币领取状态，入参={}，返回={}", data, returnStr);

            return returnStr;
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("resultInfo", "");

            LOGGER.error("查询抖音币领取状态失败：{}，错误信息={}", data, e.getMessage());
            e.printStackTrace();
        }
        return result.toString();
    }

    /**
     * 领取抖币
     */
    @DistributedLock
    @RequestMapping(value = "/draw", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject draw(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(request);
        String userid = data.getString("userid");
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");

        String smschannel = "";
        if (data.containsKey("sou")) {
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();
        }

        JSONObject result = new JSONObject();

        try {
            data.put("usermob", usermobService.decodeUserId(userid, cpid));
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("smschannel", smschannel);

            LOGGER.info("领取抖币，入参={}", data);

            result = douyinCoinService.draw(data);

        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("resultInfo", "");

            LOGGER.info("领取抖币失败，入参={}，返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return result;
    }

    /**
     * 查询账号是否正确（抖音未提供能力，查询不了）
     * @param req
     * @param response
     * @return
     */
    @RequestMapping(value = "/can", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String canJoin(HttpServletRequest req, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(req);
        String uid = data.getString("uid");
        String result = douyinCoinService.queryDouyinAccount(uid);
        return result;
    }

    @DistributedLock
    @RequestMapping(value = "/test", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject test(HttpServletRequest request, HttpServletResponse response) throws InterruptedException {
        Thread.sleep(5000);
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("resultMsg", "成功");
        return result;
    }
}