package com.chinacreator.access.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.exception.BusinessException;
import com.chinacreator.access.service.*;
import com.chinacreator.lock.redisson.annotation.DistributedLock;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.Util;

/**
 * 大转盘抽奖
 */
@RestController
@RequestMapping("/lottery")
public class LotteryController {
    private static final Logger LOGGER = LoggerFactory.getLogger(RightsController.class);

    @Autowired
    private RightsService rightsService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private LotteryService lotteryService;

    @Autowired
    private ChannelInfoService channelInfoService;

    @DistributedLock(value = "userid")
    @RequestMapping(value = "/draw", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String draw(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(request);
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        try {
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String usermob = usermobService.decodeUserId(userid, cpid, "10");

            String smschannel = "";
            if (data.containsKey("sou")) {
                String orderchannel = Util._N_to_10(data.getString("sou"), 32);
                ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
                smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();
            }

            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("usermob", usermob);
            data.put("smschannel", smschannel);

            LOGGER.info("大转盘抽奖，入参={}", data);

            result = this.lotteryService.draw(data);

        } catch (BusinessException e) {
            result.put("resultCode", e.getResultCode());
            result.put("resultInfo", e.getErrorInfo());
            result.put("errorInfo", e.getErrorInfo());

            LOGGER.info("大转盘抽奖失败，入参={}，返回={}", data, result);
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("resultInfo", "系统错误");
            result.put("errorInfo", "系统错误");

            LOGGER.info("大转盘抽奖失败，入参={}，返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return result.toString();

    }

    @DistributedLock(value = "userid")
    @RequestMapping(value = "/query", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String query(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(request);
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        try {
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String usermob = usermobService.decodeUserId(userid, cpid);

            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("usermob", usermob);

            LOGGER.info("大转盘抽奖，入参={}", data);

            result = this.lotteryService.query(data);

        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("resultInfo", "");

            LOGGER.info("大转盘抽奖失败，入参={}，返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return result.toString();

    }

    public static void main(String[] args) {
    }
}
