package com.chinacreator.access.controller;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.service.ChannelInfoService;
import com.chinacreator.access.service.UsermobService;
import com.chinacreator.access.service.WeiboMemberService;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.lock.redisson.annotation.DistributedLock;
import com.chinacreator.order.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 会员
 */
@RestController
@RequestMapping("/member")
public class MemberController {

    private final static Logger LOGGER = LoggerFactory.getLogger(MemberController.class);

    @Autowired
    private WeiboMemberService weiboMemberService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private ChannelInfoService channelInfoService;

    /**
     * 查询会员领取情况
     */
    @DistributedLock(param = "userid")
    @RequestMapping(value = "/query", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String query(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(request);
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");

        JSONObject result = new JSONObject();

        try {
            if (data.containsKey("uid") && !StringUtils.isEmpty(data.getString("uid"))) {
                String uid = data.getString("uid");
                data.put("uid", uid);
            } else {
                String userid = data.getString("userid");
                data.put("usermob", usermobService.decodeUserId(userid, cpid));
            }

            data.put("cpid", cpid);
            data.put("productid", productid);
            LOGGER.info("会员领取状态查询，入参={}", data);

            String returnStr = weiboMemberService.query(data).toString();;

            return returnStr;
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("resultInfo", "");

            LOGGER.error("会员领取状态查询失败：入参={}，返回={}", data, e.getMessage());
            e.printStackTrace();
        }
        return result.toString();
    }

    /**
     * 领取会员
     */
    @DistributedLock(param = "userid")
    @RequestMapping(value = "/draw", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject draw(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(request);
        String userid = data.getString("userid");
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");

        String smschannel = "";
        if (data.containsKey("sou")) {
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();
        }

        JSONObject result = new JSONObject();

        try {
            data.put("usermob", usermobService.decodeUserId(userid, cpid));
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("smschannel", smschannel);

            LOGGER.info("领取会员，入参={}", data);

            result = weiboMemberService.draw(data);

        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("resultInfo", "");

            e.printStackTrace();
            LOGGER.error("领取会员，入参={}，返回={}", data, e.getMessage());
        }

        return result;
    }

    /**
     * 是否可以领取新浪微博会员
     * 领取时，判断用户能否领取会员，不能领取，提示用户切换账号后领取
     * 订购时，提示用户当前账号不能领取会员，是否继续订购
     * 已通过其他方式开通包月微博会员包月的用户不能领取
     */
    @RequestMapping(value = "/can", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String canJoin(HttpServletRequest req, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToAliJSON(req);
        String uid = data.getString("uid");
        String result = weiboMemberService.canJoin(uid);
        return result;
    }
}
