package com.chinacreator.access.controller;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.restrain.ReqEnum;
import com.chinacreator.access.service.*;
import com.chinacreator.access.util.DateUtil;
import com.chinacreator.access.util.IpAddrUtil;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.lock.redisson.annotation.DistributedLock;
import com.chinacreator.order.util.DesUtilTo32;
import com.chinacreator.order.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import redis.clients.jedis.JedisCluster;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 前向产品
 * @author jiangtao
 */
@RestController
@RequestMapping("/comm/")
public class OrderController {

    private static final Logger LOGGER = LoggerFactory.getLogger(OrderController.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");

    private ExecutorService threadPool = Executors.newCachedThreadPool();

    // 订单查询服务保存的有效的订购关系
    private static final String cachePrefix = "DIRECTEDORDER:";

    @Autowired
    private WeiboOrderService weiboOrderService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private VideoRbtOrderService videoRbtOrderService;

    @Autowired
    private JedisCluster jedisCluster;

    @Autowired
    private ChannelInfoService channelInfoService;

    @Autowired
    private ChangshiPublicAccountService changshiPublicAccountService;

    /**
     * 订购
     */
    @DistributedLock(param = "userid")
    @RequestMapping(value = "/order", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String order(HttpServletRequest request, HttpServletResponse response) {

        JSONObject result = null;
        JSONObject data = TransUtil.reqToAliJSON(request);
        JSONObject dataCap = new JSONObject();

        try {
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String usermob = usermobService.decodeUserId(userid, cpid);
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();

            data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("orderchannel", Util._N_to_10(data.getString("sou"), 32));
            data.put("smschannel", smschannel);

            LOGGER.info("前向订购，入参={}", data);

            dataCap.put("usermob", usermob);
            dataCap.put("productid", productid);
            dataCap.put("channel", data.getString("sou"));
            dataCap.put("token", data.getString("token"));

            switch (productid) {
                case "2018030701":
                    // 微博畅想包
                    result = weiboOrderService.order(data);
                    break;
                case "2021011301": // 今日头条视频彩铃流量融合包（前向）
                case "2021073001": // 抖音视频彩铃定向流量月包
                case "2021073003": // 抖音视频彩铃定向流量月包（次月生效）
                case "2021091601": // 抖音视频彩铃定向流量月包（协议版）

                case "2021073002": // 抖音视频彩铃定向流量月包（联通）
                case "2021073004": // 抖音视频彩铃定向流量月包（联通次月生效）
                case "2021091602": // 抖音视频彩铃定向流量月包（联通协议版）
                case "2021091603": // 抖音视频彩铃定向流量月包（趸交版）
                case "2021091604": // 抖音视频彩铃定向流量月包（联通趸交版）

                case "2022111403": // 抖音定向流量视频彩铃5G加油包10GB
                case "2022111401": // 抖音定向流量视频彩铃5G加油包10GB

                    // 视频彩铃、抖音视频彩铃定向流量月包、抖音视频彩铃定向流量月包（次月生效）

                    result = videoRbtOrderService.order(data);
                    break;
                default:
                    // 百度等其他
                    result = orderService.order(data);
                    break;
            }

            Map<String, String> param = new HashMap<>();
            try {
                param.put("usermob", usermob);
                param.put("productid", productid);
                param.put("orderchannel", orderchannel);

                SimpleDateFormat sdf =new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                if ("0000".equals(result.getString("resultCode"))) {
                    // keyword1|productname,keyword2|orderstatus,keyword3|price,keyword4|starttime
                    param.put("busiid", "ordersuccess");
                    param.put("orderstatus", "成功");
                    param.put("starttime", sdf.format(new Date()));
                } else {
                    // keyword1|usermob,keyword2|ordertime,keyword3|productname,keyword4|orderstatus,keyword5|remark
                    param.put("busiid", "orderfail");
                    param.put("usermob", usermob);
                    param.put("ordertime", sdf.format(new Date()));
                    param.put("orderstatus", "失败");
                    param.put("remark", result.getString("errorInfo"));
                }

                threadPool.execute(() -> {
                    try {
                        changshiPublicAccountService.send(param);
                    } catch (Exception e) {
                        LOGGER.error("发送畅视公众号模板消息：参数={}，错误={}，线程={}", param, e.getMessage(), Thread.currentThread().getName());
                        throw new RuntimeException(e);
                    }
                });

            } catch (Exception e) {
                LOGGER.error("发送畅视公众号模板消息：参数={}，错误={}", param, e.getMessage());
            }

        } catch (Exception e) {
            LOGGER.error("前向订购，入参={}，返回={}", data, e.getMessage());
            LOGGER_CAP.info("产品订购，入参={}，返回={}", dataCap, e.getMessage());
            e.printStackTrace();

            data = Util.reqAliJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
            return data.toString();
        }
//        JSONObject jsonObject = JSONObject.parseObject(result);
        if ("6631".equals(result.getString("resultCode"))) {
            result.put("errorInfo", "用户已欠费，请缴费后重新发起！");
        }

        LOGGER_CAP.info("前向订购，入参={}，返回={}", dataCap, result);
        return result.toString();
    }

    /**
     * 退订
     */
    @RequestMapping(value = "/cancel", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String cancel(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = null;

        JSONObject data = TransUtil.reqToAliJSON(request);
        JSONObject dataCap = new JSONObject();

        try {
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String usermob = usermobService.decodeUserId(userid, cpid);
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();

            if (!ObjectUtils.isEmpty(usermob) && usermob.length() > 11) {
                usermob = DesUtilTo32.decode(usermob, DateUtil.date2String(new Date(), "yyyyMMdd"));
                data.put("usermob", usermob);
            }

            if (!StringUtils.isEmpty(data.getString("canceltype")) &&
                    !"undefined".equals(data.getString("canceltype"))) {
                data.put("canceltype", URLDecoder.decode(data.getString("canceltype"), "UTF-8"));
            } else {
                data.put("canceltype", "");
            }

            if (!StringUtils.isEmpty(data.getString("cancelmsg")) &&
                    !"undefined".equals(data.getString("cancelmsg"))) {
                data.put("cancelmsg", URLDecoder.decode(data.getString("cancelmsg"), "UTF-8"));
            } else {
                data.put("cancelmsg", "");
            }

            data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("orderchannel", Util._N_to_10(data.getString("sou"), 32));
            data.put("smschannel", smschannel);

            LOGGER.info("前向退订，入参={}", data);

            dataCap.put("usermob", usermob);
            dataCap.put("productid", productid);
            dataCap.put("channel", data.getString("sou"));
            dataCap.put("token", data.getString("token"));

            switch (productid) {
                case "2021091601":
                case "2021091603":
                    result = new JSONObject();
                    result.put("resultCode", "8880");
                    result.put("resultCode", "协议版产品，不支持通过H5页面退订！");
                    break;
                case "2021011301":
                case "2021073001":
                case "2021073003":
                case "2022111403": // 抖音定向流量视频彩铃5G加油包10GB
                case "2022111401": // 抖音定向流量视频彩铃5G加油包10GB

                    // 视频彩铃、抖音视频彩铃定向流量月包、抖音视频彩铃定向流量月包（次月生效）
                    result = videoRbtOrderService.cancel(data);
                    break;
                default:
                    result = orderService.cancel(data);
                    break;
            }

            Map<String, String> param = new HashMap<>();
            try {

                param.put("usermob", usermob);
                param.put("productid", productid);
                param.put("orderchannel", orderchannel);

                param.put("busiid", "cancel");

                if ("0000".equals(result.getString("resultCode"))) {
                    // tel|usermob,type|productname,date|endtime
                    param.put("endtime", DateUtil.getLastDayOfTheMonth(
                            LocalDate.now().getYear(), LocalDate.now().getMonthValue()) + " 23:59:59");

                    threadPool.execute(() -> {
                        try {
                            changshiPublicAccountService.send(param);
                            LOGGER.info("发送畅视公众号模板消息：参数={}，错误={}", param);
                        } catch (Exception e) {
                            LOGGER.error("发送畅视公众号模板消息：参数={}，错误={}", param, e.getMessage());
                        }
                    });
                } else {
                    LOGGER.info("退订失败不发送模板消息：参数={}", param);
                }


            } catch (Exception e) {
                LOGGER.error("发送畅视公众号模板消息：参数={}，错误={}", param, e.getMessage());
            }

        } catch (Exception e) {
            LOGGER.error("前向退订失败，入参={}", data, e.getMessage());
//            LOGGER_CAP.info("前向退订，入参={}，返回={}", dataCap, e.getMessage());
            e.printStackTrace();

            data = Util.reqAliJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
            return data.toString();
        }

//        LOGGER_CAP.info("前向退订，入参={}，返回={}", dataCap, result);

        return result.toString();
    }

    /**
     * 查询订购关系
     */
    @RequestMapping(value = "/queryorder", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String queryOrder(HttpServletRequest request, HttpServletResponse response) {
        JSONObject jsonObject = new JSONObject();
        String result = "";

        try {
            JSONObject data = TransUtil.reqToAliJSON(request);
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();

            if (StringUtils.isEmpty(userid)) {
                data.put("usermob", data.getString("usermob"));
            } else {
                data.put("usermob", usermobService.decodeUserId(userid, cpid));
            }

            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("smschannel", smschannel);

            LOGGER.info("订购关系查询，入参={}", data);

            switch (productid) {
                //视频彩铃
                case "2021011301":
                // 抖音视频彩铃定向流量月包
                case "2021073001":
                // 抖音视频彩铃定向流量月包（次月生效）
                case "2021073003":
                case "2022111408": // 抖音定向流量5G加油包20GB，因为支持退订状态展示
                case "2022111419": // 抖音定向流量5G加油包5GB，因为支持退订状态展示
                    String concurrentTestFlag = data.getString("concurrentTestFlag");

                    // 同一个外网IP，1小时内可以查询100次
                    String remoteIp = IpAddrUtil.getIpAddr(request);
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHH");
                    String yyyyMMddHH = sdf.format(new Date());
                    String cnt = jedisCluster.get("orderchannel:queryorder:" + remoteIp + yyyyMMddHH);

                    if (cnt == null || cnt == "") {
                        jedisCluster.setex("orderchannel:queryorder:" + remoteIp + yyyyMMddHH, 60 * 60, "0");
                    } else {
                        jedisCluster.setex("orderchannel:queryorder:" + remoteIp + yyyyMMddHH, 60 * 60, String.valueOf(Integer.valueOf(cnt) + 1));
                        LOGGER.info("remoteIp={}，cnt={}", remoteIp, cnt);
                        if (!"1".equals(concurrentTestFlag) && Integer.valueOf(cnt) > 100) {
                            jsonObject.put("resultCode", "8001");
                            jsonObject.put("errorInfo", "操作太频繁");
                            return jsonObject.toString();
                        }
                    }

                    // 只允许800.wo.cn的域名可以调用本接口
                    String url = request.getRequestURL().toString();
                    if ("1".equals(concurrentTestFlag)
                            || url.startsWith("https://800.wo.cn")
                            || url.startsWith("http://100.0.3.69")
                            || url.startsWith("http://800.wo.cn")) {

                        // 先从订单查询的redis缓存中查询用户订购关系，缓存数据缺少订购状态字段。
//                        try {
//                            String key = cachePrefix + cpid + ":" + productid + ":" + data.getString("usermob");
//                            Set<String> set = jedisCluster.smembers(key);
//
//                            LOGGER.info("抖音视频彩铃缓存：{}", set);
//                            // 为了提高查询效率，只要有数据就认为是订购状态的，这个处理可能不是很准确。
//                            // 如果用户的订购关系已经失效了，设置彩铃的时候也会提示错误
//                            if (set.size() > 0) {
//                                jsonObject.put("resultCode", "0000");
//                                return jsonObject.toString();
//                            }
//                        } catch (Exception e) {
//                            LOGGER.error("抖音视频彩铃缓存查询失败：{}", e.getMessage());
//                            e.printStackTrace();
//                        }

                        // redis没有订购关系，调用后台接口查询，查询接口响应3s左右，较慢
                        result = videoRbtOrderService.query(data);
                    } else {
                        jsonObject.put("resultCode", "8001");
                        jsonObject.put("errorInfo", "系统忙，请稍候再试");
                        LOGGER.info("订购关系查询，入参={}", jsonObject);
                        return jsonObject.toString();
                    }

                    // 同一个手机号，1小时内可以查询100次
                    // 同一个uuid，1小时内可以查询100次

                    break;
                default:
                    // 其它产品直接调用后台查询订购关系
                    result = orderService.queryOrder(data);
                    break;
            }

        } catch (Exception e) {
            e.printStackTrace();
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");
            LOGGER.info("订购关系查询，入参={}", jsonObject);
            return jsonObject.toString();
        }
        LOGGER.info("订购关系查询，入参={}", result);
        return result;
    }


}
