package com.chinacreator.access.controller;

import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.service.ChannelInfoService;
import com.chinacreator.access.service.SmsService;
import com.chinacreator.access.service.UsermobService;
import com.chinacreator.access.util.DateUtil;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.lock.redisson.annotation.DistributedLock;
import com.chinacreator.order.util.DesUtilTo32;
import com.chinacreator.order.util.Util;
import com.mysql.cj.log.Log;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;

/**
 * @author jiangtao
 */
@RestController
public class SmsController {

    private static final Logger LOGGER = LoggerFactory.getLogger(SmsController.class);

    @Autowired
    private SmsService smsService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private ChannelInfoService channelInfoService;

    @RequestMapping(value = "/sms/send", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject send(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        try {
            JSONObject data = TransUtil.reqToJSON(request);
            LOGGER.info("验证码发送，入参={}", data);

            String smstype = data.getString("smstype");

            String smschannel = "";
            if (data.containsKey("sou")) {
                LOGGER.info(data.getString("sou"));
                LOGGER.info("sou={}", Util._N_to_10(data.getString("sou"), 32));
                String orderchannel = Util._N_to_10(data.getString("sou"), 32);
                LOGGER.info("orderchannel={}", orderchannel);
                ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
                LOGGER.info("channelInfo={}", channelInfo);
                smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();
                LOGGER.info("smschannel={}", smschannel);
            }

            data.put("smschannel", smschannel);

            LOGGER.info("验证码发送2，入参={}", data);

            switch (smstype) {
                case "ordersuccess":
                case "cancelsuccess":
                    LOGGER.error("异常短信，" + data);
                    result.put("resultCode", 8888);
                    result.put("resultInfo", "短信异常");
                    return result;
                default:
                    result = smsService.send(data);
                    break;
            }
            LOGGER.info("验证码发送2，入参={}，返回={}", data, result);
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", e.getMessage());
            e.printStackTrace();
        }

        return result;

    }

    /**
     *  由于我们通过nginx通过/sms对短信发送频率做了控制，
     *  所以叫：checkvercode
     * @param response
     * @return
     */
    @DistributedLock(param = "usermob")
    @RequestMapping(value = {"/sms/check", "/checkvercode"}, method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public JSONObject check(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        result.put("resultCode", "0002");
        result.put("errorInfo", "验证码错误");

        try {
            JSONObject data = TransUtil.reqToJSON(request);
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String usermob = data.getString("usermob");
            if (usermob.length() == 32) {
                usermob = DesUtilTo32.decode(usermob, DateUtil.date2String(new Date(), "yyyyMMdd"));
                data.put("usermob", usermob);
            }

            if (StringUtils.isEmpty(data.getString("code"))) {
                result.put("errorInfo", "验证码不能为空");
                LOGGER.error("验证码校验失败，入参={}，返回={}", data, result);
                return result;
            }

            if (data.getString("code").length() < 6) {
                result.put("errorInfo", "验证码长度不正确");
                LOGGER.error("验证码校验失败，入参={}，返回={}", data, result);
                return result;
            }

            if (smsService.check(data)) {
                String userid = usermobService.encodeUserId(usermob, cpid);

                result.put("userid", userid);
                result.put("usermob", usermob.substring(0, 3) + "xxxx" + usermob.substring(8));
                result.put("resultCode", "0000");
                result.put("errorInfo", "成功");
            }
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", e.getMessage());
        }

        return result;
    }

}
