package com.chinacreator.access.controller;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.exception.BusinessException;
import com.chinacreator.access.restrain.ReqEnum;
import com.chinacreator.access.service.*;
import com.chinacreator.access.util.IpAddrUtil;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.lock.redisson.annotation.DistributedLock;
import com.chinacreator.order.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import redis.clients.jedis.JedisCluster;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 中国联通APP接口
 */
@RestController
@RequestMapping("/unicom")
public class UnicomAppController {

    private static final Logger LOGGER = LoggerFactory.getLogger(UnicomAppController.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");

    @Autowired
    private UnicomAppService unicomAppService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private OrderService orderService;

    @Autowired
    private VideoRbtOrderService videoRbtOrderService;

    @Autowired
    private JedisCluster jedisCluster;

    @Autowired
    private ChannelInfoService channelInfoService;

    /**
     * 中国联通APP免登录接口，通过ticket获取用户手机号码
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/check", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String checkTicket(HttpServletRequest request, HttpServletResponse response) {
        JSONObject result = new JSONObject();
        result.put("resultCode", "0000");
        result.put("errorInfo", "");

        JSONObject data = TransUtil.reqToAliJSON(request);

        try {
            String ticket = data.getString("ticket");
            if (StringUtils.isEmpty(ticket)) {
                result.put("resultCode", "8602");
                result.put("errorInfo", "缺少必要参数ticket!");
                return result.toString();
            }

            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);

            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("orderchannel", Util._N_to_10(data.getString("sou"), 32));
            LOGGER.info("中国联通APP免登录接口，入参={}", data);

            result = this.unicomAppService.checkTicket(data);

        } catch (BusinessException e) {
            result.put("resultCode", e.getResultCode());
            result.put("resultInfo", e.getErrorInfo());
            result.put("errorInfo", e.getErrorInfo());

            LOGGER.info("中国联通APP免登失败，入参={}，返回={}", data, result);
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("resultInfo", "系统错误");
            result.put("errorInfo", "系统错误");

            LOGGER.info("中国联通APP免登失败，入参={}，返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return result.toString();

    }

    /**
     * 手厅免登录订购接口
     * @param request
     * @param response
     * @return
     */
    @DistributedLock(param = "usermob")
    @RequestMapping(value = "/order", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String order(HttpServletRequest request, HttpServletResponse response) {

        String result = "";
        JSONObject data = TransUtil.reqToAliJSON(request);
        JSONObject dataCap = new JSONObject();

        try {
            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String usermob = usermobService.decodeUserId(userid, cpid);
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();

            data.put("usermob", usermob);
            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("orderchannel", Util._N_to_10(data.getString("sou"), 32));
            data.put("smschannel", smschannel);

            LOGGER.info("前向订购，入参={}", data);

            dataCap.put("usermob", usermob);
            dataCap.put("productid", productid);
            dataCap.put("channel", data.getString("sou"));
            dataCap.put("token", data.getString("token"));

            switch (productid) {
                case "2021073001": // 抖音视频彩铃定向流量月包
                case "2022071807": // 视频彩铃、抖音视频彩铃定向流量月包、抖音视频彩铃定向流量月包（次月生效）
                case "2022111408": // 抖音定向流量5G加油包20GB
                case "2022111419": // 抖音定向流量5G加油包5GB
                case "2018072301": // 9元头条 2023-03-15
                case "2022071811": // 30元20G
                    result = unicomAppService.order(data).toString();
                    break;
                default:
                    throw new BusinessException(ReqEnum.ERROR_Exception_aesutil_decrypt);
            }

        } catch (Exception e) {
            LOGGER.error("前向订购，入参={}，返回={}", data, e.getMessage());
            LOGGER_CAP.info("产品订购，入参={}，返回={}", dataCap, e.getMessage());
            e.printStackTrace();

            data = Util.reqAliJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
            return data.toString();
        }
        JSONObject jsonObject = JSONObject.parseObject(result);
        if ("6631".equals(jsonObject.getString("resultCode"))) {
            jsonObject.put("errorInfo", "用户已欠费，请缴费后重新发起！");
        }
        LOGGER_CAP.info("前向订购，入参={}，返回={}", dataCap, result);
        return jsonObject.toString();
    }

    /**
     * 查询订购关系
     */
    @RequestMapping(value = "/queryorder", method = RequestMethod.POST, produces = "application/json; charset=UTF-8")
    public String queryOrder(HttpServletRequest request, HttpServletResponse response) {
        JSONObject jsonObject = new JSONObject();
        String result = "";
        JSONObject data = TransUtil.reqToAliJSON(request);
        try {

            String userid = data.getString("userid");
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            String smschannel = "";
            if (data.containsKey("sou")) {
                String orderchannel = Util._N_to_10(data.getString("sou"), 32);
                ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
                smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();
            }

            if (StringUtils.isEmpty(userid)) {
                data.put("usermob", data.getString("usermob"));
            } else {
                data.put("usermob", usermobService.decodeUserId(userid, cpid));
            }

            data.put("cpid", cpid);
            data.put("productid", productid);
            data.put("smschannel", smschannel);

            LOGGER.info("订购关系查询，入参={}", data);

            switch (productid) {
                //视频彩铃
                case "2021011301":
                    // 抖音视频彩铃定向流量月包
                case "2021073001":
                    // 抖音视频彩铃定向流量月包（次月生效）
                case "2021073003":
                case "2022111408": // 抖音定向流量5G加油包20GB，因为支持退订状态展示
                case "2022111419": // 抖音定向流量5G加油包5GB，因为支持退订状态展示
                    String concurrentTestFlag = data.getString("concurrentTestFlag");

                    // 同一个外网IP，1小时内可以查询100次
                    String remoteIp = IpAddrUtil.getIpAddr(request);
                    String cnt = jedisCluster.get("orderchannel:queryorder:" + remoteIp);

                    if (cnt == null || cnt == "") {
                        jedisCluster.setex("orderchannel:queryorder:" + remoteIp, 60 * 60, "0");
                    } else {
                        if (!"1".equals(concurrentTestFlag) && Integer.valueOf(cnt) > 100) {
                            jsonObject.put("resultCode", "8001");
                            jsonObject.put("errorInfo", "操作太频繁");
                            return jsonObject.toString();
                        }
                        jedisCluster.set("orderchannel:queryorder:" + remoteIp, String.valueOf(Integer.valueOf(cnt) + 1));
                    }

                    // 只允许800.wo.cn的域名可以调用本接口
                    String url = request.getRequestURL().toString();
                    if ("1".equals(concurrentTestFlag)
                            || url.startsWith("https://800.wo.cn")
                            || url.startsWith("http://100.0.3.69")
                            || url.startsWith("http://800.wo.cn")) {

                        // 先从订单查询的redis缓存中查询用户订购关系，缓存数据缺少订购状态字段。
//                        try {
//                            String key = cachePrefix + cpid + ":" + productid + ":" + data.getString("usermob");
//                            Set<String> set = jedisCluster.smembers(key);
//
//                            LOGGER.info("抖音视频彩铃缓存：{}", set);
//                            // 为了提高查询效率，只要有数据就认为是订购状态的，这个处理可能不是很准确。
//                            // 如果用户的订购关系已经失效了，设置彩铃的时候也会提示错误
//                            if (set.size() > 0) {
//                                jsonObject.put("resultCode", "0000");
//                                return jsonObject.toString();
//                            }
//                        } catch (Exception e) {
//                            LOGGER.error("抖音视频彩铃缓存查询失败：{}", e.getMessage());
//                            e.printStackTrace();
//                        }

                        // redis没有订购关系，调用后台接口查询，查询接口响应3s左右，较慢
                        result = videoRbtOrderService.query(data);
                    } else {
                        jsonObject.put("resultCode", "8001");
                        jsonObject.put("errorInfo", "系统忙，请稍候再试");
                        LOGGER.info("订购关系查询，入参={}", jsonObject);
                        return jsonObject.toString();
                    }

                    // 同一个手机号，1小时内可以查询100次
                    // 同一个uuid，1小时内可以查询100次

                    break;
                default:
                    // 其它产品直接调用后台查询订购关系
                    result = this.unicomAppService.queryInfo(data);
                    break;
            }

        } catch (Exception e) {
            e.printStackTrace();
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");
            LOGGER.info("订购关系查询，入参={}，返回={}", data, jsonObject);
            return jsonObject.toString();
        }
        LOGGER.info("订购关系查询，入参={}，返回={}", data, result);
        return result;
    }

}
