package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import redis.clients.jedis.JedisCluster;

/**
 * 视频彩铃后向包连续包月
 *
 * @author jiangtao
 */
@Service
public class BackRbtService {

    private static final Logger LOGGER = LoggerFactory.getLogger(BackRbtService.class);

    @Value("${back.rbt.order}")
    private String payUrl;

    @Value("${back.rbt.cancel}")
    private String cancelUrl;

    @Value("${back.rbt.query}")
    private String queryUrl;

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${outOfStock.key}")
    private String key;

    @Autowired
    private JedisCluster jedisCluster;

    /**
     * 南研签约并扣费
     * <p>
     * PAY_ALIPAY_H5_WOFLOW           支付宝H5支付
     * PAY_ALIPAY_H5_ENTRUST_WOFLOW   支付宝H5签约支付
     * PAY_WECHAT_H5_WOFLOW           微信H5支付
     * PAY_WECHAT_H5_ENTRUST_WOFLOW   微信H5签约支付
     */
    public JSONObject order(JSONObject para, String requestIp) {

        String productType = para.getString("productType");
        String payMethod = para.getString("payMethod");
        String payType = "";
        switch (productType + payMethod) {
            case "11":
                payType = "PAY_WECHAT_H5_WOFLOW";
                break;
            case "12":
                payType = "PAY_ALIPAY_H5_WOFLOW";
                break;
            case "21":
                payType = "PAY_WECHAT_H5_ENTRUST_WOFLOW";
                break;
            case "22":
                payType = "PAY_ALIPAY_H5_ENTRUST_WOFLOW";
                break;
            default:
                JSONObject result = new JSONObject();
                result.put("resultCode", "8001");
                result.put("errorInfo", "支付方式参数错误");
                break;
        }

        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("usermob", para.getString("usermob"));
        param.put("client_ip", requestIp);
        param.put("dic_number", "toutiao_2020110401");
        param.put("crbt", para.getString("ringid"));
        param.put("pay_type", payType);
        param.put("channelcode", para.getString("orderchannel"));
        param.put("return_url", para.getString("returnUrl"));
        param.put("sign", SignatureUtil.signatureStr(param, key));

        String resultString = "";
        try {
            resultString = HttpClientUtil.httpClientOfPost(payUrl, param);
            LOGGER.info("后向包订购，入参={}，返回={}", param, resultString);
        } catch (Exception e) {
            LOGGER.info("后向包订购失败，入参={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);

        // 支付宝支付，服务端暂存数据
        if ("0000".equals(result.getString("resultCode"))) {
            jedisCluster.setex(result.getString("orderid"), 60 * 60 * 24, result.getString("outhtml"));
            result.put("payUrl", result.getString("outhtml"));
            result.put("orderId", result.getString("orderid"));

            result.remove("orderid");
            result.remove("outhtml");
        }

        return result;
    }

    /**
     * 退订（解约）
     */
    public JSONObject cancel(JSONObject para, String requestIp) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("usermob", para.getString("usermob"));
        param.put("client_ip", requestIp);
        param.put("dic_number", "toutiao_2020110401");
        param.put("channelcode", para.getString("orderchannel"));
        param.put("remark", "用户主动取消包月");
        param.put("sign", SignatureUtil.signatureStr(param, key));

        String resultString = "";
        try {
            resultString = HttpClientUtil.httpClientOfPost(cancelUrl, param);
            LOGGER.info("后向包订购，入参={}，返回={}", param, resultString);
        } catch (Exception e) {
            LOGGER.info("后向包订购失败，入参={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);
        return result;
    }

    /**
     * 南研状态查询
     */
    public JSONObject queryPay(JSONObject para, String requestIp) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("outOrderNo", para.getString("orderId"));
        param.put("sign", SignatureUtil.signatureStr(param, key));

        String resultString = "";
        try {
            resultString = HttpClientUtil.httpClientOfPost(queryUrl, param);
            LOGGER.info("南研支付查询，入参={}，返回={}", param, resultString);
        } catch (Exception e) {
            LOGGER.info("南研支付查询失败，入参={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);

        JSONObject data = result.getJSONObject("data");
        result.put("payStatus", data.getString("payStatus"));
        result.put("orderStatus", data.getString("orderStatus"));

        return result;
    }
}
