package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 抖音后向叠加包
 */
@Service
public class BackService {

    private static final Logger LOGGER = LoggerFactory.getLogger(StackpkgService.class);

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${outOfStock.key}")
    private String key;

    @Value("${stackpkgPay.url}")
    private String payUrl;

    @Value("${stackpkgQueryOrder.url}")
    private String queryOrderUrl;

    @Value("${stackpkgQueryPay.url}")
    private String queryPayUrl;

    @Value("${nanyan.signPay.url}")
    private String signPayUrl;

    @Value("${nanyan.cancelSign.url}")
    private String cancelSignUrl;

    /**
     * 购买
     *
     * @param para
     * @return
     */
    public JSONObject order(JSONObject para, String requestIp, String attach, String title) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("usermob", para.getString("usermob"));
        param.put("create_ip", requestIp);
        param.put("attach", attach);
        // "叠加包测试-9元10G", para.getString("productname")
        param.put("title", title);
        param.put("sign", SignatureUtil.signatureStr(param, key));
        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(payUrl, param);
            LOGGER.info("后向包订购，入参={}，返回={}", param, resultString);
        } catch (Exception e) {
            LOGGER.info("后向包订购失败，入参={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);

        return result;
    }

    /**
     * 南研微信签约与支付
     * @param para
     * @param requestIp
     * @param dicNumber
     * @return
     */
    public JSONObject nanyanSign(JSONObject para, String requestIp, String dicNumber) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("usermob", para.getString("usermob"));
        param.put("sign_ip", requestIp);
        param.put("dic_number", dicNumber);
        // 是否需要签约
        param.put("signing", para.getString("signing"));
        param.put("sign", SignatureUtil.signatureStr(param, key));
        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(signPayUrl, param);
            LOGGER.info("南研支付的后向包订购，入参={}，返回={}", param, resultString);
        } catch (Exception e) {
            LOGGER.info("南研支付的后向包订购失败，入参={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);

        return result;
    }

    /**
     * 微信解约
     * @param para
     * @param requestIp
     * @param dicNumber
     * @return
     */
    public JSONObject nanyanUnsign(JSONObject para, String requestIp, String dicNumber) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("usermob", para.getString("usermob"));
        param.put("clent_ip", requestIp);
        param.put("channelcode", "99");
        param.put("dic_number", dicNumber);
        param.put("sign", SignatureUtil.signatureStr(param, key));
        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(cancelSignUrl, param);
            LOGGER.info("南研支付的取消签约，入参={}，返回={}", param, resultString);
        } catch (Exception e) {
            LOGGER.info("南研支付的取消签约，入参={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);

        return result;
    }

    public static void main(String[] args) {

    }
}

