package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.bean.CpInfo;
import com.chinacreator.access.util.IpAddrUtil;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.util.AESUtil;
import com.chinacreator.order.util.Util;
import com.github.kevinsawicki.http.HttpRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Date;

@Service
public class BueryPointService {

    private static final Logger LOGGER = LoggerFactory.getLogger(BueryPointService.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");

    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Autowired
    private CpInfoService cpInfoService;

    @Value("${burypoint.logid.url}")
    private String logIdUrl;

    @Autowired
    private UsermobService usermobService;

    public String logid(HttpServletRequest req) throws Exception {
        JSONObject data = TransUtil.reqToAliJSON(req);
        String operid = data.getString("operid");
        String userid = data.getString("userid");
        String uuid = data.getString("uuid");
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        // String spid = Util.fogdec(data.getString("fog"), "productid");
        String productid = Util.fogdec(data.getString("fog"), "productid");

        String fescode = data.getString("fes");
        String channel = data.getString("sou");

        // fescode = MathUtil.binary32to10(fescode);
        // channel = MathUtil.binary32to10(channel);

        String operip = IpAddrUtil.getIpAddr(req);

        String remoteIp = IpAddrUtil.getIpAddr(req);
        String opertime = sdf.format(new Date());

        String url = data.getString("url");
        String remark = data.getString("remark");

        opertime = URLEncoder.encode(opertime, "UTF-8");

        try {
            if (!StringUtils.isEmpty(userid)) {
                userid = usermobService.decodeUserId(userid, cpid);
            } else {
                userid = "";
            }
        } catch (Exception t) {

        }

        JSONObject cap = new JSONObject();
        cap.put("mobile", userid);
        cap.put("type", operid);
        cap.put("seqId", uuid);
        cap.put("url", (url == null ? "" : url));
        cap.put("ip", IpAddrUtil.getIpAddr(req));

        String password = null;
        CpInfo cpInfo = cpInfoService.findCpInfo(cpid);
        password = cpInfo.getKeypwd();

        // 我们系统中配置的密码有的超过了16位，或者并且长度不是16的倍数
        // select cpid,keypwd from tb_directed_cpinfo where length(keypwd) != 16;
        if (password.length() < 16) {
            String str ="0000000000000000";
            password = password + str.substring(0, 16 - password.length());
        }

        if (password.length() % 16 != 0) {
            password = password.substring(0, password.length() - password.length()%16);
        }

        String result;
        StringBuilder sb = new StringBuilder();

        try {
            sb.append("operid=").append(operid)
                    .append("&usermob=").append(userid)
                    .append("&uuid=").append(uuid)
                    .append("&cpid=").append(cpid)
                    .append("&spid=").append(AESUtil.encrypt(productid, password))
                    .append("&fescode=").append(fescode)
                    .append("&channel=").append(channel)
                    .append("&url=").append(URLEncoder.encode((url == null ? "" : url), "UTF-8"))
                    .append("&remark=").append(URLEncoder.encode((remark == null ? "": remark), "UTF-8"))
                    .append("&ip=").append(operip)
                    .append("&opertime=").append(opertime);

            HttpRequest request = HttpRequest.get(logIdUrl + "?" + sb.toString());
            result = new String(request.bytes(), Charset.forName("UTF-8"));

            LOGGER.info("埋点，入参={}，返回={}", sb, result);

            switch (operid) {
                case "110001":
                case "310200":
                case "310201":
                case "310202":
                case "310203":
                case "310204":
                case "310205":
                case "310206":
                case "310207":
                case "310000":
                case "310001":
                case "310008":
                case "310009":
                    JSONObject resultJSON = JSONObject.parseObject(result);
                    JSONObject capJSON = new JSONObject();
                    capJSON.put("resultCode", resultJSON.getString("resultcode"));
                    capJSON.put("errorInfo", resultJSON.getString("errorinfo"));
                    LOGGER_CAP.info("页面访问，入参={}，返回={}", cap, capJSON);
                    break;
                default:
                    break;
            }

        } catch (Exception e) {
            result = "{\"resultcode\":\"8000\",\"errorinfo\":\"调用埋点接口失败\"}";
            LOGGER.error("埋点，入参={}，返回={}，IP={}", sb, e.getMessage(), remoteIp);

            // LOGGER_CAP.error("页面访问PV保存失败，入参={}，返回={}", cap, e.getMessage());
            e.printStackTrace();
        }


        return result;
    }

}
