package com.chinacreator.access.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import com.chinacreator.order.util.Util;

/**
 * 多选一产品的订购
 * @author jiangtao
 */
@Service
public class ChoiceOrderService {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(ChoiceOrderService.class);

	@Value("${outOfStock.channel}")
    private String channel;

    @Value("${outOfStock.key}")
    private String key;
	
	@Value("${choiceOrder.url}")
	private String choiceOrderUrl;
	
	@Value("${choiceCancel.url}")
	private String choiceCancelUrl;

	/**
	 * 订购
	 *
	 */
	public JSONObject order(JSONObject data) {
		
		JSONObject inmap = new JSONObject();
		inmap.put("channel", channel);
		inmap.put("orderchannel", Util._N_to_10(data.getString("sou"), 32));
		inmap.put("usermob", data.getString("usermob"));
		inmap.put("token", data.getString("token"));
		inmap.put("twoip", data.getString("requestIp"));
		// 2020021201订购百度9元包,2018072301订购头条9元包,2020071301预约订购百度9元包,2020071401预约订购头条9元包
		inmap.put("productid", data.getString("productid"));
		inmap.put("sign", SignatureUtil.signatureStr(inmap, key));
		
		String resultString = "";
		JSONObject result = new JSONObject();
        try {
			resultString = HttpClientUtil.httpClientOfPost(choiceOrderUrl, inmap);
            result = JSONObject.parseObject(resultString);

			LOGGER.info("多选一产品订购，入参={}，返回={}", inmap, resultString);

        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", "系统忙，请稍候再试");

			LOGGER.error("多选一产品订购失败，入参={}，返回={}", data, e.getMessage());
			e.printStackTrace();
        }

        return result;
	}

	/**
	 * 退订
	 *
	 */
	public JSONObject cancel(JSONObject data) {
		JSONObject inmap = new JSONObject();
		inmap.put("channel", channel);
		inmap.put("orderchannel", Util._N_to_10(data.getString("sou"), 32));
		inmap.put("usermob", data.getString("usermob"));
		inmap.put("twoip", data.getString("requestIp"));
		// 2020021201退订百度9元包,2018072301退订头条9元包
		inmap.put("productid", data.getString("productid"));
		inmap.put("sign", SignatureUtil.signatureStr(inmap, key));
		
		String resultString = "";
		JSONObject result = new JSONObject();
        try {
            resultString = HttpClientUtil.httpClientOfPost(choiceCancelUrl, inmap);
            result = JSONObject.parseObject(resultString);

			LOGGER.info("多选一产品退订，入参={}，返回={}", inmap, result);
        } catch (Exception e) {
            result.put("resultCode", "8000");
            result.put("errorInfo", "系统忙，请稍候再试");

			LOGGER.error("多选一产品退订失败，入参={}，返回={}", data, e.getMessage());
			e.printStackTrace();
        }

        return result;
	}
}
