package com.chinacreator.access.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.bean.FesMessage;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SerialnumberUtil;
import com.chinacreator.order.bean.SendBean;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.AESUtil;
import com.chinacreator.order.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * 文明办APP免流产品订购
 */
@Service
public class CivilizedService {

    private static final Logger LOGGER = LoggerFactory.getLogger(CivilizedService.class);
    public static HttpSendService send = new HttpSendServicImp();

    private PartService partService;

    private SmsService smsService;

    @Value("${civilize.cpid}")
    private String cpid;

    @Value("${civilize.password}")
    private String password;

    @Value("${civilize.orderUrl}")
    private String orderUrl;

    /**
     * 发送短信
     */
    @Value("${sendUrl}")
    private String sendUrl;

    @Autowired
    public void setPartService(PartService partService) {
        this.partService = partService;
    }

    @Autowired
    public void setSmsService(SmsService smsService) {
        this.smsService = smsService;
    }

    public JSONObject order(JSONObject data) {
        JSONObject jsonObject = new JSONObject();

        try {
            SendBean sendBean = null;

            Map<String, Object> inmap = new HashMap<>();
            Map<String, String> datamap = new HashMap<>();
            datamap.put("usermob", data.getString("usermob"));
            datamap.put("productid", data.getString("productid"));
            datamap.put("channel", data.getString("orderchannel"));
            datamap.put("orderid", SerialnumberUtil.Getnum(100000, 999999));
            inmap.put("cpid", cpid);
            inmap.put("data", AESUtil.encrypt(JSON.toJSONString(datamap), password));

            String resultString = HttpClientUtil.jsonOfPost(orderUrl, JSON.toJSONString(inmap));
            LOGGER.info("订购结果，接口地址={}, 入参={}，加密后={}, 返回={}", orderUrl, inmap, datamap, resultString);
            jsonObject = JSONObject.parseObject(resultString);

            //下发成功短信
            if ("0000".equals(jsonObject.get("resultCode"))) {
                int sou = Integer.parseInt(Util._N_to_10(data.getString("sou"), 32));
                String fescod = Util._N_to_10(data.getString("fes"), 32);

                // 用户在前一天打开页面，获取了getstyle，但是在转钟后才提交请求，那么活动结束，需要新的短信
                FesMessage fes = partService.fesExists(fescod, sou);
                if (fes != null) {
                    data.put("new_fescod", fes.getFescod());
                }

                sendBean = Util.retSmsBean(data, 0);
                JSONObject result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

                LOGGER.info("发送订购成功短信，入参={}, 返回={}", data, result);
            } else {
                net.sf.json.JSONObject paraData = new net.sf.json.JSONObject();
                paraData.put("usermob", data.getString("usermob"));
                paraData.put("fog", data.getString("fog"));
                paraData.put("smstype", "orderfail");
                paraData.put("errorInfo", jsonObject.get("errorInfo"));
                paraData.put("smschannel", data.getString("smschannel"));

                net.sf.json.JSONObject smsResult = this.smsService.send(paraData);
                LOGGER.info("发送订购失败短信，入参={}, 返回={}", data, smsResult.toString());
            }
        } catch (Exception e) {
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");
            LOGGER.error("订购失败，入参={}, 返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return jsonObject;
    }



}
