package com.chinacreator.access.service;

import java.nio.charset.Charset;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.github.kevinsawicki.http.HttpRequest;

/**
 * 抖音币权益
 */
@Service
public class DouyinCoinService {

    private static final Logger LOGGER = LoggerFactory.getLogger(DouyinCoinService.class);

    public static HttpSendService send = new HttpSendServicImp();

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${sinaKey}")
    private String key;

    @Value("${secret}")
    private String secret;

    /**
     * 查询新浪会员的领取状态
     */
    @Value("${douyinCoinQuery.url}")
    private String douyinCoinQueryUrl;

    @Value("${douyinCoinDraw.url}")
    private String douyinCoinDrawUrl;

    @Value("${douyinAccount.url}")
    private String douyinAccountUrl;

    @Value("${sendUrl}")
    private String sendUrl;

    @Autowired
    private UsermobService usermobService;

    /**
     * 领取抖音币
     *
     * @param data
     * @return
     */
    public JSONObject draw(JSONObject data) throws Exception {


        JSONObject param = new JSONObject();
        // 接口分配
        param.put("channel", channel);
        //抖音号
        param.put("customer_id", data.getString("uid"));
        //号码
        param.put("usermob", data.getString("usermob"));
        param.put("activity_id", "20200310");
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        long startTime = System.currentTimeMillis();
        String result = HttpClientUtil.httpClientOfPost(douyinCoinDrawUrl, param);

        long endedTime = System.currentTimeMillis();
        LOGGER.info("领取抖币，入参={}，返回={}，耗时={}s", param, result, (endedTime - startTime)/1000);

        // JSONObject jsonObject = JSONObject.parseObject(result);

        // 下发成功短信
        // 志强那边发送了短信
        //if ("0000".equals(jsonObject.getString("resultCode"))) {
            // SendBean sendBean = Util.retSmsBean(data, 211);
            // send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));
        //}

        return JSONObject.parseObject(result);
    }

    /**
     * 查询号码是否领取斗币或者斗币领取是否成功
     */
    public JSONObject query(JSONObject data) {
        JSONObject param = new JSONObject();
        // 接口分配
        param.put("channel", channel);
        param.put("usermob", data.getString("usermob"));
        param.put("activity_id", "20200310");
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        long startTime = System.currentTimeMillis();
        String result = HttpClientUtil.httpClientOfPost(douyinCoinQueryUrl, param);

        long endedTime = System.currentTimeMillis();
        LOGGER.info("查询领取状态查询，入参={}，返回={}，耗时={}s", result, (endedTime - startTime)/1000);

        JSONObject jsonObject = JSONObject.parseObject(result);
        // if (!jsonObject.containsKey("user_mob")) {
        //     jsonObject.put("user_mob", "18674842761");
        // }

        String usermob = jsonObject.getString("usermob");

        jsonObject.put("usermob", usermob.substring(0, 3) + "xxxx" + usermob.substring(7, 11));

        return jsonObject;
    }

    /**
     * 查询账号是否存在，账号是否领取抖音币
     *
     * @param uid 抖音账号
     * @return
     */
    public String queryDouyinAccount(String uid) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        // 抖音号 dyjfo6acb201
        param.put("search_ids", uid);
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        long startTime = System.currentTimeMillis();
        HttpRequest request = HttpRequest.post(douyinAccountUrl).form(param);
        String result = new String(request.bytes(), Charset.forName("UTF-8"));

        long endedTime = System.currentTimeMillis();
        result = result.replaceAll("resultInfo", "errorInfo");

        LOGGER.info("抖音账号校验，入参={}，返回={}，耗时={}s", result, (endedTime - startTime)/1000);

        return result;
    }

}
