package com.chinacreator.access.service;

import com.alibaba.fastjson.JSON;
import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.bean.FesMessage;
import com.chinacreator.access.dao.MessageDao;
import com.chinacreator.access.dao.OrderMapper;
import com.chinacreator.access.restrain.ReqEnum;
import com.chinacreator.access.util.DateUtil;
import com.chinacreator.access.util.SerialnumberUtil;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.bean.DirectedJyLogBean;
import com.chinacreator.order.bean.DirectedJyOrderBean;
import com.chinacreator.order.bean.SendBean;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.AESUtil;
import com.chinacreator.order.util.Util;
import net.sf.json.JSONObject;
import org.apache.commons.ssl.KeyMaterial;
import org.apache.commons.ssl.SSLClient;
import org.apache.commons.ssl.TrustMaterial;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.util.*;
import java.util.Map.Entry;

/**
 * @author Yk
 */
@Service
public class DxService {

    private static final Logger LOGGER = LoggerFactory.getLogger(DxService.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");

    private static final String requestUrl = Util.getsendUrl();
    private static HttpSendService send = new HttpSendServicImp();

    @Value("${jksPassword}")
    private String jksPassword;

    @Value("${pathTojksFile}")
    private String pathToJksFile;

    @Autowired
    private MessageDao messagedao;

    @Autowired
    private OrderService orderService;

    @Autowired
    private PartService partService;

    private static SSLClient client;

    @Autowired
    private OrderMapper orderMapper;

    @PostConstruct
    public void postInit() {
        try {
            client = new SSLClient();
            client.addTrustMaterial(TrustMaterial.DEFAULT);
            client.addTrustMaterial(new KeyMaterial(pathToJksFile, jksPassword.toCharArray()));
            // default setting is "true" for
            client.setCheckHostname(true);
            // default setting is "true" forclient.setCheckCRL(true);
            // default setting is "true" for SSLClient
            client.setCheckExpiry(false);
            client.setWantClientAuth(true);
            // Let's load a client certificate (max: 1 per SSLClient instance).
            client.setKeyMaterial(new KeyMaterial(pathToJksFile, jksPassword.toCharArray()));

        } catch (GeneralSecurityException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 对定向提交数据(经过统一认证方)
     *
     * @param data 请求数据包装的Json对象
     * @param i    0订购 1退订
     * @return 返回结果Json
     * @throws Exception
     */
    public JSONObject submitSc(JSONObject data, int i) throws Exception {
        JSONObject result = new JSONObject();
        //发起订购 or 退订
        SendBean sendBean = Util.retOrderBean(data, i);
        FesMessage fesMessage = null;
        JSONObject param = JSONObject.fromObject(sendBean);

        String productId = Util.fogdec(data.getString("fog"), "productid");

        // 抖音视频彩铃定向流量包（协议版）不可以通过页面进行退订
        if (productId.equals("2021091601") || productId.equals("2021091603")) {
            result.put("resultCode", "8880");
            result.put("resultCode", "协议版产品，不支持通过H5页面退订！");
            return result;
        }

        result = send.sendPost(requestUrl, param);

        LOGGER.info("前向产品订购，入参={}，返回={}", JSON.toJSONString(sendBean), result);

        // 保存订购关系
        try {
            DirectedJyOrderBean directedJyOrderBean = new DirectedJyOrderBean();
            String orderid = SerialnumberUtil.Getnum(10000000,99999999);
            directedJyOrderBean.setOrderid(orderid);
            directedJyOrderBean.setCpid(data.getString("cpid"));
            if (i == 0) {
                directedJyOrderBean.setBusiid("unOrder");
                directedJyOrderBean.setOrderchannel(Util._N_to_10(data.getString("sou"), 32));
            } else if (i == 1) {
                directedJyOrderBean.setBusiid("cancel");
                directedJyOrderBean.setCancelchannel(Util._N_to_10(data.getString("sou"), 32));
            } else if (i == 4) {
                directedJyOrderBean.setBusiid("order");
                directedJyOrderBean.setOrderchannel(Util._N_to_10(data.getString("sou"), 32));
            }

            directedJyOrderBean.setUsermob(data.getString("usermob"));
            directedJyOrderBean.setProductid(data.getString("productid"));
            directedJyOrderBean.setCancelchannel(data.getString("cancelchannel"));
            directedJyOrderBean.setCanceltime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));
            directedJyOrderBean.setCurstatus("1");

            orderMapper.cancelOrder(directedJyOrderBean);
        }catch (Exception ef){
            ef.printStackTrace();
        }

        //记录日志表
        try {
            DirectedJyLogBean logBean = new DirectedJyLogBean();
            logBean.setOrderid(SerialnumberUtil.Getnum(10000000,99999999));
            logBean.setCpid(data.getString("cpid"));
            if (i == 0) {
                logBean.setBusiid("unOrder");
                logBean.setOrderchannel(Util._N_to_10(data.getString("sou"), 32));
            } else if (i == 1) {
                logBean.setBusiid("cancel");
                logBean.setCancelchannel(Util._N_to_10(data.getString("sou"), 32));
            } else if (i == 4) {
                logBean.setBusiid("order");
                logBean.setOrderchannel(Util._N_to_10(data.getString("sou"), 32));
            }
            logBean.setUsermob(data.getString("usermob"));
            logBean.setProductid(data.getString("productid"));
            logBean.setOrderchannel(data.getString("orderchannel"));
            logBean.setRequestparam(com.alibaba.fastjson.JSONObject.toJSONString(sendBean));
            logBean.setResponsecode(result.get("resultCode")==null?"":result.get("resultCode").toString());
            logBean.setResponseinfo(result.get("errorInfo")==null?"":result.get("errorInfo").toString());
            logBean.setRequesttime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));

            orderMapper.insertDirectedJyLog(logBean);
        } catch (Exception ef){
            ef.printStackTrace();
        }

        if (!"0000".equals(result.getString("resultCode"))) {
            return result;
        }

        try {
            //下发成功短信
            //String fog = data.getString("fog");
            //String fes = data.getString("fes");
            // String cpid = Util.fogdec(fog, "cpid");
            // String sou = data.getString("sou");

            //春节活动
            /*if (("9uq".equals(fes) && ("2h".equals(sou) || "1g".equals(sou)
                                    || "1h".equals(sou)  || "15".equals(sou)
                                    || "47".equals(sou)  || "2k".equals(sou)
                                    || "18".equals(sou)  || "4a".equals(sou)
                                    || "12".equals(sou) )) ||
                    ("9on".equals(fes) && "1m".equals(sou))

                ) {
                // 客户端页面做活动20191206 - 20191212
                JSONObject json = new JSONObject();
                json.put("busiid", "sms");
                json.put("usermob", data.getString("usermob"));
                json.put("smstype", "910");
                json.put("smscontent", "");
                SendBean sbean = new SendBean();
                sbean.setCpid(cpid);
                sbean.setData(json);

                send.sendPost(requestUrl, JSONObject.fromObject(sbean));
            } else {
                send.sendPost(requestUrl, JSONObject.fromObject(Util.retSmsBean(data, i)));
            }*/

            // 助力武汉、短彩、海报渠道、青海短信推广
            /*if ("9on".equals(fes) && ("1m".equals(sou) || "4d".equals(sou) || "2l".equals(sou) || "4g".equals(sou))) {
                JSONObject json = new JSONObject();
                json.put("busiid", "sms");
                json.put("usermob", data.getString("usermob"));
                json.put("smstype", "911");
                json.put("smscontent", "");
                SendBean sbean = new SendBean();
                sbean.setCpid(cpid);
                sbean.setData(json);

                send.sendPost(requestUrl, JSONObject.fromObject(sbean));
            } else {
                send.sendPost(requestUrl, JSONObject.fromObject(Util.retSmsBean(data, i)));
            }*/
            String fog = data.getString("fog");
            String fes = data.getString("fes");
            int sou = Integer.valueOf(Util._N_to_10(data.getString("sou"), 32));
            String fescod = Util._N_to_10(data.getString("fes"), 32);
            // 用户在前一天打开页面，获取了getstyle，但是在转钟后才提交请求，那么活动结束，需要新的短信
            fesMessage = partService.fesExists(fescod, sou);

            if (fes != null) {
                data.put("new_fescod", fesMessage.getFescod());
            }

            param = JSONObject.fromObject(Util.retSmsBean(data, i));
            JSONObject sendResult = send.sendPost(requestUrl, param);

            LOGGER.info("前向产品" + (i == 0 ? "订购" : "退订") + "发送成功短信，入参={}，返回={}，活动={}", param, sendResult, fesMessage);

            if (i == 0) {
                LOGGER_CAP.info("前向产品"+(i==0?"订购":"退订") + "发送成功短信，入参={}，返回={}，活动={}", param, sendResult, fesMessage);
            }

            if (!"2017060800064876".equals(Util.fogdec(fog, "cpid"))) {
                //进行数据同步	头条不会做订购退订同步，头条业务只有激活，订购/退订由定时任务执行
                activateSc(data, i);
            }
        } catch (Exception e) {
            //订购流程中下发短信与数据同步出错，不覆盖订购结果
            LOGGER.error("前向产品"+(i==0?"订购":"退订")+"发送成功短信失败，入参={}，返回={}，活动={}", param, e.getMessage(), fesMessage);
        }

        return result;
    }

    /**
     * 预约
     *
     * @param data
     * @return
     * @throws Exception
     */
    public JSONObject yuyueSc(JSONObject data) throws Exception {
        ArrayList<Object> paratype = new ArrayList<Object>();
        paratype.add(Util.fogdec(data.getString("fog"), "cpid"));
        paratype.add(data.getString("usermob"));
        paratype.add(Util.fogdec(data.getString("fog"), "productid"));
        ArrayList<JSONObject> produ = messagedao.queryProductid(paratype);
        //此用户未订购相关产品
        if (produ.size() == 0) {
            return Util.reqJsonErr(ReqEnum.No_acti_null);
        }

        if (produ.get(0).getString("usertype").equals("4")) {//判断号码是否为4G号码
            if (Util.bcode(data)) {//判断验证码
                data.put("fog", "df3958be828815ec18613612fa1e7a55ec0c66c954ceccf62ada29431f927b3f5f7bae7134a9f80704eb805a47704037");
                JSONObject cancelJson = send.sendPost(requestUrl, JSONObject.fromObject(Util.retOrderBean(data, 1)));
                String resultCode = cancelJson.getString("resultCode");
                if (resultCode.equals("0000")) {
                    //判断用户退订是否成功  成功后需要存入用户信息到 预约订购表 后续定时任务订购9元包
                    data.put("productid", "2018072301");//加入9元产品编码
                    //存入用户号码,预约时间,预约产品cpid,预约后产品编码,预约用户的设备号
                    String sql = "INSERT INTO tb_ordercenter_yuyue(usermob,cpid,productid,useraccount,sou,ytime) VALUES(?,?,?,?,?,SYSDATE())";
                    ArrayList<Object> para = new ArrayList<Object>();
                    para.add(data.getString("usermob"));
                    para.add(Util.fogdec(data.getString("fog"), "cpid"));
                    para.add(data.getString("productid"));
                    para.add(data.getString("useraccount"));
                    para.add(Util._N_to_10(data.getString("sou"), 32));
                    boolean boo = messagedao.insert(sql, para);
                    if (boo) {//预约信息存入成功
                        //发送预约成功短信
                        JSONObject codejsonhf = new JSONObject();
                        codejsonhf.put("usermob", data.getString("usermob"));
                        codejsonhf.put("content", "尊敬的用户您好，您已成功退订头条15元6GB定向流量包，且为您预约升级今日头条9元10GB定向流量包，预约成功，确保手机及信誉正常情况下，下月初以收到的升级成功短信为准。");
                        Util.sendVerCode(codejsonhf);
                        return Util.reqJsonErr(ReqEnum.OK_yuyue);
                    } else {
                        return Util.reqJsonErr(ReqEnum.No_yuyue_inset);
                    }
                } else {
                    //预约订购原有套餐退订失败
                    return Util.reqJsonErr(ReqEnum.No_yuyue_cancel);
                }
            } else {
                return Util.reqJsonErr(ReqEnum.ERROR_No_code_found);
            }
        } else {
            return Util.reqJsonErr(ReqEnum.No_yuyue_type);
        }
    }

    /**
     * 实时激活入口(与对应厂家同步订购关系);
     * 当订购成功时有需要激活的厂家同步关系至客户.
     * 当用户到激活页面时先调用订购关系查询服务确认在本地存在订购关系在同步设备号至对应客户
     *
     * @param data
     * @return
     * @throws Exception 0 同步有设备号是自动激活  1退订 2激活
     *                   此接口沿用了原有头条用于同步数据时使用的接口
     *                   当传入状态为0 且存在设备号时 为激活 用于在头条页面上订购成功后直接激活
     *                   当传入状态为1时 为激活用户
     */
    public JSONObject activateSc(JSONObject data, int i) throws Exception {
        LOGGER.info("激活操作：入参={}, 标记(0同步1退订2激活)={}", data.toString(), i);

        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String usermob = data.getString("usermob");
        String productid = Util.fogdec(data.getString("fog"), "productid");

        //拿取本次请求的cpid与电话号码
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(cpid);
        para.add(usermob);
        para.add(productid);

        ArrayList<JSONObject> product;
        switch (Util.fogdec(data.getString("fog"), "cpid")) {
            case "2017060800064876":
                //头条
                if (i == 2) {
                    //判断激活的验证码
                    if (!Util.bcode(data)) {
                        return Util.reqJsonErr(ReqEnum.ERROR_No_code_found);
                    }

                    product = messagedao.queryProductid(para);
                    if (product.size() == 0) {
                        LOGGER.info("激活失败，用户未订购本产品：" + data.toString());
                        return Util.reqJsonErr(ReqEnum.No_acti_null);
                    }

                    data.put("sid", product.get(0).get("orderid"));
                    data.put("fog", Util.fogret(data.getString("fog"), product.get(0).getString("productid")));
                    data.put("productid", product.get(0).getString("productid"));
                    data.put("ordertime", product.get(0).getString("ordertime"));
                    data.put("canceltime", product.get(0).getString("canceltime"));

                    String key = Util.getkeyMap(Util.fogdec(data.getString("fog"), "cpid"));

                    String arg = "";
                    JSONObject fond = new JSONObject();
                    try {
                        Integer stu = product.get(0).getInt("stu");
                        arg = AESUtil.encrypt(Util.retactivateJsonString(data, stu), key);
                        LOGGER.info("开始激活前同步：入参={}, arg={}", data, arg);
                        String ret = TransUtil.post(Util.getTtSyncUrl(), arg, client);
                        LOGGER.info("激活前同步结果：入参={}, arg={}, 返回={}", data, arg, ret);

                        arg = AESUtil.encrypt(Util.retactivateJsonString(data, i), key);
                        LOGGER.info("开始激活：入参={}, arg={}", data, arg);
                        ret = TransUtil.post(Util.getTtSyncUrl(), arg, client);
                        LOGGER.info("激活结果：入参={}, arg={}, 返回={}", data, arg, ret);

                        fond = JSONObject.fromObject(ret);

                    } catch (Exception e) {
                        LOGGER.info("激活失败：入参={}, arg={}, 返回={}", data, arg, e.getMessage());
                        e.printStackTrace();
                    }

                    if ("0000".equals(fond.getString("resultCode"))) {
                        //临时添加更新uid
                        try {
                            String sql = "UPDATE td_directed_order SET useraccount = ? WHERE cpid = ? AND productid = ? AND usermob = ?";
                            ArrayList<Object> date = new ArrayList<Object>();
                            date.add(data.getString("useraccount"));
                            date.add(Util.fogdec(data.getString("fog"), "cpid"));
                            date.add(product.get(0).getString("productid"));
                            date.add(data.getString("usermob"));
                            messagedao.booleanUid(sql, date);

                            LOGGER.info("更新用户信息成功：" + data.getString("usermob") + "--" + data.getString("useraccount") + "--i" + i);
                        } catch (Exception e) {
                            e.printStackTrace();
                            LOGGER.error("更新用户信息失败:" + data.getString("usermob") + "--" + data.getString("useraccount") + "--i" + i, e);
                        }
                    }

                    return fond;

                }

                return Util.reqJsonErr(ReqEnum.No_tongbu_sou);

            case "2019042259387282"://互娱
                //互娱需要在页面订购，所有的同步与开通需要在活动服务中处理
                if (i == 2) {//互娱乐激活
                    if (!Util.bcode(data)) {//判断激活的验证码
                        return Util.reqJsonErr(ReqEnum.ERROR_No_code_found);
                    }

                    product = messagedao.queryProductid(para);
                    if (product.size() == 0) {
                        return Util.reqJsonErr(ReqEnum.No_acti_null);
                    }
                    data.put("sid", product.get(0).get("orderid"));
                    data.put("fog", Util.fogret(data.getString("fog"), product.get(0).getString("productid")));
                    data.put("productid", product.get(0).getString("productid"));
                    data.put("ordertime", product.get(0).getString("ordertime"));
                    data.put("canceltime", product.get(0).getString("canceltime"));

                    String url = Util.getWySyncUrl();
                    String wykey = Util.getkeyMap(Util.fogdec(data.getString("fog"), "cpid"));
                    String sendStr = retactivateJsonString(data, i, wykey);
                    String result = null;
                    try {
                        result = TransUtil.post(url, sendStr);
                    } finally {
                        LOGGER.info("|activateSc| 入参={}，加密后={}，返回={}", data, sendStr, result);
                    }

                    JSONObject wyfond = JSONObject.fromObject(result);
                    if ("0000".equals(wyfond.getString("resultCode"))) {
                        //临时添加更新uid
                        try {
                            String sql = "UPDATE td_directed_order SET useraccount = ? WHERE cpid = ? AND productid = ? AND usermob = ?";
                            ArrayList<Object> date = new ArrayList<Object>();
                            date.add(data.getString("useraccount"));
                            date.add(Util.fogdec(data.getString("fog"), "cpid"));
                            date.add(product.get(0).getString("productid"));
                            date.add(data.getString("usermob"));

                            LOGGER.error("更新用户信息成功:" + data.getString("usermob") + "--" + data.getString("useraccount") + "--i" + i);
                            messagedao.booleanUid(sql, date);
                        } catch (Exception e) {
                            LOGGER.error("更新用户信息失败:" + data.getString("usermob") + "--" + data.getString("useraccount") + "--i" + i, e);
                        }
                    }
                    return wyfond;
                }

                return Util.reqJsonErr(ReqEnum.ERROR_No_data_found);
            default:
                return Util.reqJsonErr(ReqEnum.No_tongbu_sou);
        }
    }

    public String retactivateJsonString(JSONObject data, int i, String key) throws Exception {
        JSONObject json = new JSONObject();
        json.put("sid", data.getString("sid"));
        json.put("usermob", data.getString("usermob"));
        json.put("productid", data.getString("productid"));
        json.put("useraccount", data.getString("useraccount"));
        json.put("curstatus", i);//状态

        SortedMap<String, String> parameters = new TreeMap<String, String>();
        parameters.put("sid", data.getString("sid"));
        parameters.put("usermob", data.getString("usermob"));
        parameters.put("productid", data.getString("productid"));
        parameters.put("useraccount", data.getString("useraccount"));
        parameters.put("curstatus", Integer.valueOf(i).toString());
        if (i == 0) {//同步订购
            json.put("ordertime", data.getString("ordertime"));
            parameters.put("ordertime", data.getString("ordertime"));
            json.put("canceltime", data.getString("canceltime"));
        }
        if (i == 1) {//退订
            json.put("ordertime", data.getString("ordertime"));
            json.put("canceltime", data.getString("canceltime"));
            parameters.put("canceltime", data.getString("canceltime"));
        }
        if (i == 2) {//号码激活不存在订购时间与退订时间
            json.put("ordertime", data.getString("ordertime"));
            json.put("canceltime", data.getString("canceltime"));
            parameters.put("ordertime", data.getString("ordertime"));
            parameters.put("canceltime", data.getString("canceltime"));
        }
        String channel = Util._N_to_10(data.getString("sou"), 32);
        json.put("channel", channel);
        parameters.put("channel", channel);
        json.put("sign", createSign("UTF-8", parameters, key));
        return json.toString();
    }

    /**
     * 1.所有参与传参的参数按照accsii排序（升序）
     * 2.空值不传递，不参与签名组串
     * 3.排序后结果加上key=key（参数）
     * 与常用Sign有区别。
     *
     * @param characterEncoding
     * @param parameters
     * @param key
     * @return
     */
    public String createSign(String characterEncoding, SortedMap<String, String> parameters, String key) {
        StringBuffer sb = new StringBuffer();
        StringBuffer sbkey = new StringBuffer();
        Set<Entry<String, String>> es = parameters.entrySet();//所有参与传参的参数按照accsii排序（升序）
        Iterator<Entry<String, String>> it = es.iterator();
        while (it.hasNext()) {
            Map.Entry<String, String> entry = it.next();
            String k = entry.getKey();
            Object v = entry.getValue();
            //空值不传递，不参与签名组串
            if (null != v && !"".equals(v)) {
                sb.append(k + v);
                sbkey.append(k + v);
            }
        }
        sbkey = sbkey.append("key=" + key);
        LOGGER.info("|createSign| 加密的字符串：" + sbkey);
        String sign = "";
        try {//加密
            sign = AESUtil.encrypt(sbkey.toString(), key);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return sign;
    }

    /**
     * 获取用户手机号 头条设备号获取
     *
     * @param data
     * @return
     * @throws Exception
     */
    public JSONObject devToPhone(JSONObject data) throws Exception {
        String device = data.getString("device_id");
        System.out.println(Util.gettouTiaoToPhone() + device);
        LOGGER.info("获取手机号地址:" + Util.gettouTiaoToPhone() + device);
        return send.sendPost(Util.gettouTiaoToPhone() + device, data);
    }

    /**
     * 退订接口  用于退订用户多产品(同一客户)
     * TODO 退订可以不需要判断多用户应在查询流量时才有退订 查询流量时已知道用户当前订购的产品
     * @param data 请求数据包装的Json对象
     * @param i    0订购 1退订
     * @return 返回结果Json
     * @throws Exception
     */
    public JSONObject cancelSc(JSONObject data, int i) throws Exception {
        if (Util.bcode(data)) {//判断验证码
            ArrayList<Object> para = new ArrayList<Object>();
            para.add(Util.fogdec(data.getString("fog"), "cpid"));
            para.add(data.getString("usermob"));
            para.add(Util.fogdec(data.getString("fog"), "productid"));

            ArrayList<JSONObject> date = messagedao.queryProductid(para);//根据用户手机号,与cpid查询用户库中存在的订购关系
            if (date.size() == 1) {//判断是否取出了用户的产品id
                JSONObject fond = send.sendPost(requestUrl, JSONObject.fromObject(Util.retOrderBean(data, date.get(0).getString("productid"), i)));
                if (fond.get("resultCode").equals("0000")) {
                    send.sendPost(requestUrl, JSONObject.fromObject(Util.retSmsBean(data, date.get(0).getString("productid"), i)));
                    data.put("fog", Util.fogret(data.getString("fog"), date.get(0).getString("productid")));
                    activateSc(data, i);
                }
                return fond;
            } else {
                return Util.reqJsonErr(ReqEnum.ERROR_No_ordr_found);
            }
        } else {
            return Util.reqJsonErr(ReqEnum.ERROR_No_code_found);
        }
    }


    /**
     * 查询流量接口根据客户cpid查询用户对应流量接口
     *
     * @param data 请求数据包装的Json对象 0订购  1退订
     * @return 返回结果Json
     * @throws Exception
     */
    public JSONObject cpidAndPhoneToFolw(JSONObject data) throws Exception {
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(Util.fogdec(data.getString("fog"), "cpid"));
        para.add(data.getString("usermob"));
        para.add(Util.fogdec(data.getString("fog"), "productid"));
        ArrayList<JSONObject> date = messagedao.queryProductid(para);
        //根据用户手机号,与cpid查询用户库中存在记录,且是否还在订购状态
        if (date.size() == 1) {
            //判断是否取出了用户的产品id
            JSONObject fond = send.sendPost(requestUrl, JSONObject.fromObject(Util.retFlowBean(data, date.get(0).getString("productid"))));
            if (fond.get("resultCode").equals("0000")) {
                String da =
                        AESUtil.decrypt(fond.get("queryflow").toString(),
                                Util.keyMap.get(
                                        Util.fogdec(data.getString("fog"), "cpid")
                                )
                        );
                fond.put("queryflow", da);
                JSONObject retDate = date.get(0);
                retDate.put("phone", data.getString("usermob").substring(0, 3) + "****" + data.getString("usermob").substring(7, 11));
                fond.put("date", retDate);
            }
            return fond;
        } else {
            return Util.reqJsonErr(ReqEnum.ERROR_No_ordr_found);
        }
    }

    public void taskYuyue() throws Exception {
        // 1.查询出预约表中数据
        ArrayList<JSONObject> yuyuedate = messagedao.queryYuyue();
        String sql = "UPDATE tb_ordercenter_yuyue"
                + " SET num = num+1,stat = ?,otime = NOW()"
                + " WHERE usermob = ? AND cpid = ? AND productid = ?";
        for (JSONObject jsonObject : yuyuedate) {
            JSONObject fond = send.sendPost(requestUrl, JSONObject.fromObject(Util.retYuYueOrderBean(jsonObject)));
            ArrayList<Object> para = new ArrayList<Object>();
            if (fond.getString("resultCode").equals("0000") || fond.getString("resultCode").equals("6610") || fond.getString("resultCode").equals("6622")) {
                para.add("1");
            } else {
                para.add("0");
            }
            para.add(jsonObject.getString("usermob"));
            para.add(jsonObject.getString("cpid"));
            para.add(jsonObject.getString("productid"));
            messagedao.insert(sql, para);
            //订购成功的用户需要发送短信
            if (fond.getString("resultCode").equals("0000")) {
                JSONObject codejsonhf = new JSONObject();
                codejsonhf.put("usermob", jsonObject.getString("usermob"));
                codejsonhf.put("content", "尊敬的用户您好，您预约升级的今日头条9元10GB定向流量包已经升级成功，免流还包括：头条新闻、西瓜视频、火山小视频、抖音视频。订购成功后立即生效次月自动续订，感谢您的支持。流量使用详情，请咨询联通客服：10010。");
                Util.sendVerCode(codejsonhf);
            } else {//订购失败且次数为4次(当期又一次订购所以为5次)
                if (jsonObject.getString("num").equals("4")) {
                    JSONObject codejsonhf = new JSONObject();
                    codejsonhf.put("usermob", jsonObject.getString("usermob"));
                    codejsonhf.put("content", "尊敬的用户您好，您预约升级今日头条9元10GB定向流量包，预约失败，可能是由于欠费、黑名单、系统超时、省份系统等原因造成，如有疑问请咨询联通客服: 10010。");
                    Util.sendVerCode(codejsonhf);
                }
            }
        }
        //表移动
        String ydSql = "INSERT INTO tb_ordercenter_yuyue_his SELECT * FROM tb_ordercenter_yuyue WHERE num = 5 OR stat = 1";
        messagedao.insert(ydSql, null);
        String deSql = "DELETE FROM tb_ordercenter_yuyue WHERE num = 5 OR stat = 1";
        messagedao.insert(deSql, null);
    }

    /**
     * 活动页面临时日报
     *
     * @param data
     * @throws Exception
     */
    public JSONObject qylodsql(JSONObject data) throws Exception {
        String sql =
                "SELECT dgzs,fqzs FROM "
                        + "( "
                        + "SELECT COUNT(ordmob) AS dgzs FROM "
                        + "( "
                        + "SELECT ord.usermob AS ordmob FROM td_directed_order AS ord WHERE "
                        + "( "
                        + "orderchannel = 70 OR "
                        + "orderchannel = 73 OR "
                        + "orderchannel = 71 OR "
                        + "orderchannel = 75 OR "
                        + "orderchannel = 78 OR "
                        + "orderchannel = 79 OR "
                        + "orderchannel = 95 OR "
                        + "orderchannel = 96 OR "
                        + "orderchannel = 97 OR "
                        + "orderchannel = 98 OR "
                        + "orderchannel = 86 OR "
                        + "orderchannel = 87 OR "
                        + "orderchannel = 88 OR "
                        + "orderchannel = 89 "
                        + ") "
                        + "AND ord.cpid = '2017060800064876' "
                        + "AND ord.productid = '2018072301' "
                        + "AND ord.ordertime >= ? "
                        + "AND ord.ordertime <= ? "
                        + "GROUP BY ord.usermob "
                        + ") AS ordgr "
                        + ") AS t1, "
                        + "( "
                        + "SELECT COUNT(logmob) AS fqzs FROM "
                        + "( "
                        + "SELECT log.usermob AS logmob FROM tl_directed_log AS log "
                        + "WHERE "
                        + "( "
                        + "log.orderchannel = 70 OR "
                        + "log.orderchannel = 73 OR "
                        + "log.orderchannel = 71 OR "
                        + "log.orderchannel = 75 OR "
                        + "log.orderchannel = 78 OR "
                        + "log.orderchannel = 79 OR "
                        + "log.orderchannel = 95 OR "
                        + "log.orderchannel = 96 OR "
                        + "log.orderchannel = 97 OR "
                        + "log.orderchannel = 98 OR "
                        + "log.orderchannel = 86 OR "
                        + "log.orderchannel = 87 OR "
                        + "log.orderchannel = 88 OR "
                        + "log.orderchannel = 89 "
                        + ") "
                        + "AND ( log.busiid = 'order' OR log.busiid = 'unOrder' ) "
                        + "AND log.requesttime >= ? "
                        + "AND log.requesttime <= ? "
                        + "GROUP BY log.usermob "
                        + ") AS loggr "
                        + ") AS t2";
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(data.get("st"));
        para.add(data.get("en"));
        para.add(data.get("st"));
        para.add(data.get("en"));
        return messagedao.ribao(sql, para);
    }
}
