package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;


@Service
public class LotteryService {

    private static Logger LOGGER = LoggerFactory.getLogger(LotteryService.class);

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${secret}")
    private String secret;

    @Value("${lottery.drawUrl}")
    private String drawUrl;

    @Value("${lottery.queryUrl}")
    private String queryUrl;

    /**
     * 抽奖
     * @param data
     * @return
     */
    public JSONObject draw(JSONObject data) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("mark", "1001");
        param.put("product_id",data.getString("productid"));
        param.put("user_mob", data.getString("usermob"));
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        long startTime = System.currentTimeMillis();
        String result = HttpClientUtil.httpClientOfPost(drawUrl, param);

        long endedTime = System.currentTimeMillis();
        LOGGER.info("大转盘抽奖结果，入参={}，出参={}，用时={}s", param, result, (endedTime - startTime)/1000);

        return JSONObject.parseObject(result);
    }


    /**
     * 查询中奖结果
     * @param data
     * @return
     */
    public JSONObject query(JSONObject data) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("mark", "1001");
        param.put("product_id", data.getString("productid"));
        param.put("user_mob", data.getString("usermob"));
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        long startTime = System.currentTimeMillis();
        String result = HttpClientUtil.httpClientOfPost(queryUrl, param);

        long endedTime = System.currentTimeMillis();
        LOGGER.info("查询中奖结果，入参={}，出参={}，用时={}s", param, result, (endedTime - startTime)/1000);

        return JSONObject.parseObject(result);

    }
}
