package com.chinacreator.access.service;

import com.chinacreator.access.bean.FesMessage;
import com.chinacreator.access.dao.OrderMapper;
import com.chinacreator.access.util.DateUtil;
import com.chinacreator.access.util.SerialnumberUtil;
import com.chinacreator.order.bean.DirectedJyLogBean;
import com.chinacreator.order.bean.DirectedJyOrderBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import com.chinacreator.order.bean.SendBean;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.AESUtil;
import com.chinacreator.order.util.Util;

/**
 * @author jiangtao
 */
@Service
public class OrderService {

    private final static Logger LOGGER = LoggerFactory.getLogger(OrderService.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");

    public static HttpSendService send = new HttpSendServicImp();

    @Value("${sendUrl}")
    private String sendUrl;

    @Value("${fValueUrl}")
    private String fValueUrl;

    @Value("${secret}")
    private String secret;

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${orderInv.url}")
    private String orderInvUrl;

    @Autowired
    private PartService partService;

    @Autowired
    private OrderMapper orderMapper;

    public JSONObject order(JSONObject data) {
        JSONObject jsonObject = new JSONObject();
        JSONObject cap = new JSONObject();
        cap.put("cpid", data.getString("cpid"));
        cap.put("productid", data.getString("productid"));
        cap.put("usermob", data.getString("usermob"));

        try {
            SendBean sendBean = Util.retOrderBean(data, 0);
            jsonObject = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

            LOGGER.info("订购结果，入参={}, 返回={}", data, jsonObject);

            JSONObject result = null;
            //下发成功短信
            if ("0000".equals(jsonObject.get("resultCode"))) {

                DirectedJyOrderBean directedJyOrderBean = new DirectedJyOrderBean();
                String orderid = SerialnumberUtil.Getnum(10000000,99999999);
                directedJyOrderBean.setOrderid(orderid);
                directedJyOrderBean.setCpid(cap.getString("cpid"));
                directedJyOrderBean.setBusiid("order");
                directedJyOrderBean.setUsermob(cap.getString("usermob"));
                directedJyOrderBean.setProductid(cap.getString("productid"));
                directedJyOrderBean.setOrderchannel(data.getString("orderchannel"));
                directedJyOrderBean.setCurstatus("0");
                directedJyOrderBean.setOrdertime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));
                try {
                    orderMapper.insertDirectedJyOrder(directedJyOrderBean);
                }catch (Exception ef){
                    ef.printStackTrace();
                }
                // 阿里云产品，同步成功以后给用户发送短信。同步失败的用户，等重试成功后给用户发送短信
                String cpid = data.getString("cpid");
                String aliCpId = "2022071882950391";
                if (!aliCpId.equals(cpid) || (aliCpId.equals(cpid)&& "1".equals(jsonObject.get("syncstatus")))) {
                    int sou = Integer.valueOf(Util._N_to_10(data.getString("sou"), 32));
                    String fescod = Util._N_to_10(data.getString("fes"), 32);

                    // 用户在前一天打开页面，获取了getstyle，但是在转钟后才提交请求，那么活动结束，需要新的短信
                    FesMessage fes = partService.fesExists(fescod, sou);
                    if (fes != null) {
                        data.put("new_fescod", fes.getFescod());
                    }

                    sendBean = Util.retSmsBean(data, 0);
                    result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

                    LOGGER.info("发送订购成功短信，入参={}, 返回={}", data, result);
                    LOGGER_CAP.info("发送订购成功短信，入参={}, 返回={}", cap, result);
                }
            }

            //记录日志表
            DirectedJyLogBean logBean = new DirectedJyLogBean();
            logBean.setOrderid(SerialnumberUtil.Getnum(10000000,99999999));
            logBean.setCpid(cap.getString("cpid"));
            logBean.setBusiid("order");
            logBean.setUsermob(cap.getString("usermob"));
            logBean.setProductid(cap.getString("productid"));
            logBean.setOrderchannel(cap.getString("orderchannel"));
            logBean.setRequestparam(JSONObject.toJSONString(sendBean));
            logBean.setResponsecode(jsonObject.get("resultCode")==null?"":jsonObject.get("resultCode").toString());
            logBean.setResponseinfo(jsonObject.get("errorInfo")==null?"":jsonObject.get("errorInfo").toString());
            logBean.setRequesttime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));
            try {
                orderMapper.insertDirectedJyLog(logBean);
            }catch (Exception ef){
                ef.printStackTrace();
            }
        } catch (Exception e) {
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");

            LOGGER.error("订购失败，入参={}, 返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return jsonObject;
    }

    public JSONObject cancel(JSONObject data) {
        JSONObject jsonObject = null;

        try {
            SendBean sendBean = Util.retOrderBean(data, 1);
            jsonObject = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

            LOGGER.info("退订结果，入参={}, 返回={}", data, jsonObject);
            JSONObject result = null;
            if ("0000".equals(jsonObject.get("resultCode"))) {
                //下发成功短信

                DirectedJyOrderBean directedJyOrderBean = new DirectedJyOrderBean();
                String orderid = SerialnumberUtil.Getnum(10000000,99999999);
                directedJyOrderBean.setOrderid(orderid);
                directedJyOrderBean.setCpid(data.getString("cpid"));
                directedJyOrderBean.setBusiid("cancel");
                directedJyOrderBean.setUsermob(data.getString("usermob"));
                directedJyOrderBean.setProductid(data.getString("productid"));
                directedJyOrderBean.setCancelchannel(data.getString("cancelchannel"));
                directedJyOrderBean.setCanceltime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));
                directedJyOrderBean.setCurstatus("1");
                try {
                    orderMapper.cancelOrder(directedJyOrderBean);
                }catch (Exception ef){
                    ef.printStackTrace();
                }
                int sou = Integer.valueOf(Util._N_to_10(data.getString("sou"), 32));
                String fescod = Util._N_to_10(data.getString("fes"), 32);

                // 用户在前一天打开页面，获取了getstyle，但是在转钟后才提交请求，那么活动结束，需要新的短信
                FesMessage fes = partService.fesExists(fescod, sou);
                if (fes != null) {
                    data.put("new_fescod", fes.getFescod());
                }

                sendBean = Util.retSmsBean(data, 1);
                result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

                LOGGER.info("发送退订短信，入参={}, 返回={}", data, result);
//                LOGGER_CAP.info("发送退订短信，入参={}, 返回={}", data, result);
            }

            //记录日志表
            DirectedJyLogBean logBean = new DirectedJyLogBean();
            logBean.setOrderid(SerialnumberUtil.Getnum(10000000,99999999));
            logBean.setCpid(data.getString("cpid"));
            logBean.setBusiid("cancel");
            logBean.setUsermob(data.getString("usermob"));
            logBean.setProductid(data.getString("productid"));
            logBean.setOrderchannel(data.getString("orderchannel"));
            logBean.setRequestparam(JSONObject.toJSONString(sendBean));
            logBean.setResponsecode(jsonObject.get("resultCode")==null?"":jsonObject.get("resultCode").toString());
            logBean.setResponseinfo(jsonObject.get("errorInfo")==null?"":jsonObject.get("errorInfo").toString());
            logBean.setRequesttime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));
            try {
                orderMapper.insertDirectedJyLog(logBean);
            }catch (Exception ef){
                ef.printStackTrace();
            }
        } catch (Exception e) {
            jsonObject = new JSONObject();

            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");

            LOGGER.error("退订失败，入参={}, 返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return jsonObject;
    }

    /**
     * 查询订购状态
     */
    public String queryOrder(JSONObject data) throws Exception {
        String productid = Util.fogdec(data.getString("fog"), "productid");
        String cpid = Util.fogdec(data.getString("fog"), "cpid");

        data.put("productid", productid);
        data.put("cpid", cpid);

        SendBean sendBean = Util.queryOrderBean(data);
        JSONObject result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

        if ("0000".equals(result.getString("resultCode"))) {
            String encryptDsta = result.getString("data");
            String decryptData = AESUtil.decrypt(encryptDsta, Util.keyMap.get(cpid));

            result.put("data", decryptData);
        }

        //记录日志表
        DirectedJyLogBean logBean = new DirectedJyLogBean();
        logBean.setOrderid(SerialnumberUtil.Getnum(10000000,99999999));
        logBean.setCpid(data.getString("cpid"));
        logBean.setBusiid("query");
        logBean.setUsermob(data.getString("usermob"));
        logBean.setProductid(data.getString("productid"));
        logBean.setRequestparam(JSONObject.toJSONString(sendBean));
        logBean.setResponsecode(result.get("resultCode")==null?"":result.get("resultCode").toString());
        logBean.setResponseinfo(result.get("errorInfo")==null?"":result.get("errorInfo").toString());
        logBean.setRequesttime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));
        try {
            orderMapper.insertDirectedJyLog(logBean);
        }catch (Exception ef){
            ef.printStackTrace();
        }
        return result.toString();
    }

    /**
     * 订购送10G后向流量，异步订购，每分钟只能订购1次
     */
    public void orderinversion(String usermob, String activityid) {
        //String url = "http://localhost:4552/activity/repeat/orderInv";
        //JSONObject param = TransUtil.reqToAliJSON(request);
        // Map<String,Object> inmap=new HashMap<>();

        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("usermob", usermob);//号码
        param.put("activityid", activityid);//活动编码
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        try {
            String result = HttpClientUtil.httpClientOfPost(orderInvUrl, param);
            LOGGER.info("2020年4月活动订购抖音赠送10G定向流量成功, {}, {}", param, result);

        } catch (Exception e) {

            LOGGER.info("2020年4月活动订购抖音赠送10G定向流量失败, {}, {}", param, e.getMessage());
            e.printStackTrace();
        }
    }

    public static void main(String[] args) {
//        int sou = Integer.valueOf(Util._N_to_10(data.getString("sou"), 32));
//        String fescod = Util._N_to_10(data.getString("fes"), 32);
//
//        // 用户在前一天打开页面，获取了getstyle，但是在转钟后才提交请求，那么活动结束，需要新的短信
//        FesMessage fes = partService.fesExists(fescod, sou);
//        if (fes != null) {
//            data.put("new_fescod", fes.getFescod());
//        }
//
//        sendBean = Util.retSmsBean(data, 1);
//        JSONObject result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));
//
//        LOGGER.info("发送退订短信，入参={}, 返回={}", data, result);
    }

}
