package com.chinacreator.access.service;

import com.chinacreator.access.bean.FesMessage;
import com.chinacreator.access.dao.MessageDao;
import com.chinacreator.access.restrain.ReqEnum;
import com.chinacreator.access.task.GetStyleTask;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.DesUtilTo32;
import com.chinacreator.order.util.Util;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import redis.clients.jedis.JedisCluster;

import javax.annotation.PostConstruct;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author jiangtao
 */
@Service
public class PartService {

    private final static Logger LOGGER = LoggerFactory.getLogger(PartService.class);

    //private static String requestUrl = "http://127.0.4.1:9090/sms";
    private static String requestUrl = "http://100.0.4.1:9090/sms";
    private static String varSmsKey = "12345678";

    private static List<FesMessage> festivals = new ArrayList();
    private static List<JSONObject> fesChanns = new ArrayList<>();
    private static List<JSONObject> products = new ArrayList();

    @Autowired
    private JedisCluster jedisCluster;

    @PostConstruct
    public void init() {
        GetStyleTask thread = new GetStyleTask(this);
        new Thread(thread).start();
    }

    /**
     * 用于发送特定短信
     *
     * @param data  Json格式 参数: usermob 电话  / content 放松内容
     * @return 结果Json   {"resultcode":"0","resultinfo":"成功"}
     */

    HttpSendService send = new HttpSendServicImp();
    @Autowired
    private MessageDao messagedao;

    /**
     * 判断产品组是否符合
     * 获取内存中存储的页面配置信息(内存中没有数据时去库中取出数据存入缓存中);
     *
     * @param data 页面传递过来的数据
     * @return 页面样式信息
     */
    public JSONObject pageStyle(JSONObject data) {
        Integer sou = 0;
        String fescod = "";
        String n32sou = (String) data.get("sou");
        String n32fes = (String) data.get("fes");
        if (n32sou == null || n32fes == null) {
            //必要参数未传入
            LOGGER.error("链接参数校验失败，入参={}，返回={}", data, ReqEnum.ERROR_Exception_aesutil_decrypt);
            return Util.reqJsonErr(ReqEnum.ERROR_Exception_aesutil_decrypt);
        }

        try {
            sou = Integer.valueOf(n32sou, 32);
            fescod = Util._N_to_10(n32fes, 32);

            FesMessage fes = fesExists(fescod, sou);
            if (fes == null) {
                //获取聚合页面标题信息//判断活动组是否存在,是否在活动时间内
                LOGGER.error("链接参数校验失败，入参={}，返回={}", data, ReqEnum.ERROR_No_festival_found);
                return Util.reqJsonErr(ReqEnum.ERROR_No_festival_found);
            }

            if (!souExists(fes.getFescod(), sou)) {
                //渠道号未找到
                LOGGER.error("链接参数校验失败，入参={}，返回={}", data, ReqEnum.ERROR_No_sou_found);
                return Util.reqJsonErr(ReqEnum.ERROR_No_sou_found);
            }

            //获取活动组对应信息
            //获取聚合页面标题图标信息
            JSONObject reqjson = new JSONObject();
            //获取聚合页面标题信息
            reqjson.put("title", fes.getTitle());
            reqjson.put("icon", fes.getIcon());
            reqjson.put("fescod", fes.getFescod());
            reqjson.put("fes", fes.getFes());

            if (data.containsKey("orderId")) {
                reqjson.put("payUrl", jedisCluster.get(data.getString("orderId")));
            }

            if (!fescod.equals(fes.getFescod())) {
                String url = fes.getUrl();
                if (!StringUtils.isEmpty(url)) {
                    if (url.indexOf("?") > 0) {
                        url = url.substring(0, url.indexOf("?"));
                    }
                    url += "?fes=" + fes.getFes() + "&sou=" + n32sou;

                    reqjson.put("url", url);
                }
            }


            ArrayList<JSONObject> result = getPageIni(fes.getFescod(), reqjson);
            LOGGER.info("链接参数校验，入参={}，返回={}", data, result);

            //获取活动 产品组信息
            return Util.reqJson(ReqEnum.OK, reqjson, result);

        } catch (Exception e) {
            //活动未找到或不在活动时间内
            LOGGER.error("链接参数校验失败，入参={}，返回={}", data, e.getMessage());
            e.printStackTrace();

            return Util.reqJsonErr(ReqEnum.ERROR_No_festival_found);
        }
    }

    private FesMessage getFesMessageInfo(String fescod) {
        for (FesMessage fesMessage : festivals) {
            if (fescod.equals(fesMessage.getFescod())) {
                return fesMessage;
            }
        }
        return null;
    }

    /**
     * 获取活动信息
     *
     * @param fescod 活动组编码
     * @return
     * @throws SQLException
     * @throws Exception
     */
    public FesMessage fesExists(String fescod, int sou) throws SQLException, ParseException {
        if (StringUtils.isEmpty(fescod)) {
            return null;
        }
        Date date = new Date();

        FesMessage fes = getFesMessageInfo(fescod);
        if (fes == null) {
            return null;
        }

        // paritycode 不为空，表示这个活动是促销活动
        FesMessage promo = findPromoFestival(fes, date, sou);
        if (promo != null) {
            return promo;
        }

        if (checkDuration(date, fes)) {
            return fes;
        }

        FesMessage parity = findParityFestival(fes, date, sou);
        if (parity != null) {
            return parity;
        }

        return null;
    }

    /**
     * 判断当前活动渠道号是否正常
     *
     * @param fescod 发送过来的数据其中有 fescod 活动编码,sou 渠道编码 sou渠道为0线上时不需要判断
     * @return
     */
    public boolean souExists(String fescod, int sou) {
        if (StringUtils.isEmpty(fescod) || StringUtils.isEmpty(sou)) {
            return false;
        }

        for (JSONObject object : fesChanns) {
            if (fescod.equals(object.getString("fescod")) &&
                    sou == object.getInt("orderchannel")) {
                return true;
            }
        }

        FesMessage fesMessage = getFesMessageInfo(fescod);
        // 如果是全渠道，平价活动的全部渠道都可以订购
        if ("1".equals(fesMessage.getAllchannel())) {
            for (JSONObject object : fesChanns) {
                if (fesMessage.getParitycode().equals(object.getString("fescod")) &&
                        sou == object.getInt("orderchannel")) {
                    return true;
                }
            }
        }

        return false;
    }

    private FesMessage findPromoFestival(FesMessage parent, Date date, int sou) throws ParseException, SQLException {
        for (FesMessage fesMessage : festivals) {
            if (parent.getFescod().equals(fesMessage.getParitycode())) {
                if (checkDuration(date, fesMessage)) {
                    if (souExists(fesMessage.getFescod(), sou)) {
                        return fesMessage;
                    }
                }
            }
        }

        return null;
    }

    private FesMessage findParityFestival(FesMessage child, Date date, int sou) throws ParseException, SQLException {
        for (FesMessage fesMessage : festivals) {
            if (fesMessage.getFescod().equals(child.getParitycode())) {
                if (checkDuration(date, fesMessage)) {
                    if (souExists(fesMessage.getFescod(), sou)) {
                        return fesMessage;
                    }
                }
            }
        }

        return null;
    }

    /**
     * 判断 @Param date 是否在 @Param FesMessage 的开始与结束时间范围内
     * @return
     */
    private boolean checkDuration(Date date, FesMessage fesMessage) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
        Date stime = sdf.parse(fesMessage.getStime().toString() + " 00:00:00");
        Date etime = sdf.parse(fesMessage.getEtime().toString() + " 23:59:59");

        return ((date.after(stime) || date.equals(stime)) &&
                (date.before(etime) || date.equals(etime)));
    }

    public void refreshFestival() throws SQLException {
        String sql = "SELECT id,title,fescod,stime,etime,icon,paritycode,url,allchannel, conv(fescod, 10, 32) fes FROM tb_ordercenter_festival";
        List<FesMessage> temp = messagedao.queryMessageList(sql, null);

        if (temp.size() > 0) {
            festivals = temp;
        }
    }

    /**
     * 获取所有活动组信息
     *
     * @return
     * @throws SQLException
     */
    public ArrayList<FesMessage> getAllFesMessage() throws SQLException {
        String sql = "SELECT id,title,fescod,stime,etime,icon,paritycode,url,allchannel, conv(fescod, 10, 32) fes FROM tb_ordercenter_festival";
        ArrayList<FesMessage> fesMes = messagedao.queryMessageList(sql, null);
        return fesMes;
    }

    public void refreshFesChan() throws SQLException {
        long millis = System.currentTimeMillis();
        String sql = "SELECT fescod,orderchannel FROM "
                + "tb_ordercenter_fes_chan AS a,tb_ordercenter_channel AS b "
                + "WHERE  a.channelid = b.id";

        ArrayList<JSONObject> temp = messagedao.queryFesChans(sql, null);
        if (temp.size() > 0) {
            fesChanns = temp;
        }
    }

    /**
     * 获取对应活动编码的产品组信息
     *
     * @return fog
     * @throws Exception
     */
    public ArrayList<JSONObject> getPageIni(String fescod, JSONObject reqjson) throws Exception {
        /*if (products == null || products.size() == 0) {
            String sql = "SELECT cpid,productid,productname,url,icon,smstype,smsord,smscan "
                    + "FROM tb_ordercenter_product WHERE "
                    + "stime <= CURRENT_DATE AND etime >= CURRENT_DATE"
                    + " AND  fescod = ?";
            ArrayList<Object> para = new ArrayList<>();
            para.add(fescod);
            return messagedao.queryProductJson(sql, para);
        } else {*/
        ArrayList<JSONObject> result = new ArrayList<>();

        for (JSONObject object : products) {
            if (fescod.equals(object.getString("fescod"))) {
                result.add(object);
            }
        }

        return result;
        //}
    }

    public void refreshProduct() throws Exception {
        String sql = "SELECT cpid,productid,productname,url,icon,smstype,smsord,smscan,fescod,conv(fescod,10,32) fes "
                + "FROM tb_ordercenter_product WHERE "
                + "stime <= CURRENT_DATE AND etime >= CURRENT_DATE";
        ArrayList<Object> para = new ArrayList<>();
        List<JSONObject> temp = messagedao.queryProductJson(sql, para);
        if (temp.size() > 0) {
            products = temp;
        }
    }

    public JSONObject sendVerCode(JSONObject data) {
        JSONObject json = new JSONObject();
        json.put("busiid", "sendsms");
        json.put("spcode", "106555061");
        JSONObject content = new JSONObject();
        content.put("SPNumber", "106555061");
        content.put("UserNumber", data.getString("usermob"));
        content.put("ReportFlag", "1");
        content.put("MessageContent", data.getString("content"));
        String con = "";
        try {
            con = DesUtilTo32.encode(content.toString(), varSmsKey);
        } catch (Exception e) {

            e.printStackTrace();
            return Util.reqJsonErr(ReqEnum.ERROR_Exception_aesutil_decrypt);
        }
        json.put("content", con);
        JSONObject result = send.sendPost(requestUrl, json);

        LOGGER.error("发送短信验证码（sendVerCode），入参={}，返回={}", data, result);

        result.remove("SequenceNumber");
        return result;
    }

    /**
     * 判断渠道用户是否正确
     *
     * @param data
     * @return
     */
    public JSONObject querysou(JSONObject data) {
        int sou = Integer.valueOf(Util._N_to_10(data.getString("sou"), 32));
        String fescod = Util._N_to_10(data.getString("fes"), 32);

        if (!souExists(fescod, sou)) {
            LOGGER.error("渠道参数校验失败，入参=fes:{},sou={}，返回={}", fescod, sou, ReqEnum.ERROR_No_sou_found);
            return Util.reqJsonErr(ReqEnum.ERROR_No_sou_found);
        }

        JSONObject Json = new JSONObject();
        Json.put("resultCode", "0000");
        Json.put("errorInfo", sou);
        return Json;
    }

    /**
     * 返回抽奖信息,其中处理各类奖品赠送
     *
     * @param data
     * @return
     */
    public JSONObject award(JSONObject data) {
        //获取活动组编码
        String fescod = Util._N_to_10((String) data.get("fes"), 32);

        JSONObject bool = queryPrizeMass(data.getString("usermob"), fescod);

        JSONObject retjson = new JSONObject();

        if (bool == null) {//判断用户是否以抽取过奖品 //TODO
            //获取对应活动组编码奖品列表与对应奖品中奖概率
            ArrayList<JSONObject> pri = null;
            try {
                pri = getPrize(fescod);
            } catch (Exception e) {
                e.printStackTrace();
            }
            //生成抽奖概率
            int rand = Util.randomSection(1, 10000);

            JSONObject netjson = new JSONObject();
            if (pri == null) {
                throw new IllegalArgumentException("The pri cannot be null");
            }
            //概率匹配中奖
            for (JSONObject jsonObject : pri) {
                if (rand < jsonObject.getInt("prop") && 10 < jsonObject.getInt("cou")) {//奖品概率  奖品剩余数量保证奖品数量
                    netjson = jsonObject;
                    break;
                }
            }
            System.out.println(netjson.toString());
            int prizeid = netjson.getInt("prizeid");//奖品ID
            String roll = "";
            int id = 0;
//			String cpid = "";
//			String productid = "";
            //获取对应奖券编码
            try {
                JSONObject prize = queryPrize(prizeid);
                roll = prize.getString("roll");
                id = prize.getInt("id");
                //cpid = prize.getString("cpid");
                //productid = prize.getString("productid");
            } catch (Exception e) {
                LOGGER.error("", ReqEnum.ERROR_Exception_db_sql);
                e.printStackTrace();
            }
            switch (netjson.getInt("pattern")) {//奖品类型
                case 0:
                    System.out.println("奖券类型!");
                    //获取奖券
                    JSONObject codejson = new JSONObject();
                    codejson.put("usermob", data.getString("usermob"));
                    //终结者
                    String content = "恭喜您！抽中[" + netjson.getString("prizename") + "],兑换码：" + roll + ",请登录客户端兑换卷码~";
                    netjson.remove("prizename");
                    netjson.put("prizename", content);
                    codejson.put("content", content);
                    if (sendVerCode(codejson).getInt("resultcode") == 0) {//下发是否成功
                        //更新剩余库存
                        updatePrizeStock(prizeid);
                        //更新奖卷状态
                        updatePrize(id);
                        //存入中奖用户号码,时间,与奖券
                        JSONObject json = new JSONObject();
                        json.put("usermob", data.getString("usermob"));
                        json.put("prizeid", prizeid);
                        json.put("roll", roll);//奖券信息
                        json.put("fes", fescod);
                        if (insetPrizeMessage(json)) {//奖品流程走完
                            System.out.println("奖品流程走完");
                            retjson.put("resultCode", "0000");
                            retjson.put("prizename", netjson.getString("prizename"));
                            retjson.put("li", netjson.getInt("li"));
                        }
                    } else {
                        return Util.reqJsonErr(ReqEnum.ERROR_Exception_request_decrypt);
                    }
                    break;
                case 1:
                    System.out.println("0元包类型!");
                    //orderApply
                    break;
                case 2:
                    System.out.println("话费!");
                    JSONObject jsonhf = new JSONObject();
                    jsonhf.put("usermob", data.getString("usermob"));
                    jsonhf.put("prizeid", prizeid);
                    jsonhf.put("roll", netjson.getString("prizename"));//奖券信息
                    jsonhf.put("fes", fescod);
                    insetPrizeMessage(jsonhf);

                    retjson.put("resultCode", "0000");
                    retjson.put("prizename", "恭喜您！抽中[" + netjson.getString("prizename") + "],下月续订用户，在次月月初7个工作日内可到账查询，请耐心等待哦~");
                    retjson.put("li", netjson.getInt("li"));

                    JSONObject codejsonhf = new JSONObject();
                    codejsonhf.put("usermob", data.getString("usermob"));
                    codejsonhf.put("content", "恭喜您！抽中[" + netjson.getString("prizename") + "],下月续订用户，在次月月初7个工作日内可到账查询，请耐心等待哦~");
                    sendVerCode(codejsonhf);//下发短信
                    break;
                case 3:
                    System.out.println("流量!");
                    JSONObject jsonlow = new JSONObject();
                    jsonlow.put("usermob", data.getString("usermob"));
                    jsonlow.put("prizeid", prizeid);
                    jsonlow.put("roll", netjson.getString("prizename"));//奖券信息
                    jsonlow.put("fes", fescod);
                    insetPrizeMessage(jsonlow);//存储中奖信息

                    retjson.put("resultCode", "0000");
                    retjson.put("prizename", "恭喜您！抽中[" + netjson.getString("prizename") + "],下月续订用户，在次月月初7个工作日内可到账查询，请耐心等待哦~");
                    retjson.put("li", netjson.getInt("li"));

                    JSONObject codejsonll = new JSONObject();
                    codejsonll.put("usermob", data.getString("usermob"));
                    codejsonll.put("content", "恭喜您！抽中[" + netjson.getString("prizename") + "],下月续订用户，在次月月初7个工作日内可到账查询，请耐心等待哦~");
                    sendVerCode(codejsonll);//下发短信
                    break;
                default:

                    break;
            }
            return retjson;
        } else {
            //用户重复抽奖
            System.out.println("用户重复抽奖");
            retjson.put("resultCode", "8088");
            retjson.put("prizename", "您之前已抽中过奖品,奖品为:" + bool.getString("prizename") + "~");
            retjson.put("li", bool.getInt("li"));
            return retjson;
        }
    }

    /**
     * 获取对应产品奖品信息  奖品ID 奖品名称  奖品中奖概率 转盘对应弧度
     *
     * @return fescod
     * @throws Exception
     */
    public ArrayList<JSONObject> getPrize(String fescod) throws Exception {
        String sql =
                "SELECT prizeid,prizename,prop,li,pattern,cou FROM tb_ordercenter_fes_pri AS a  LEFT JOIN "
                        + "tb_ordercenter_prizeline AS b ON b.id = a.prizeid WHERE a.fescod = ? ORDER BY prop ";
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(fescod);
        return messagedao.queryPrizeJson(sql, para);
    }

    public boolean insetPrizeMessage(JSONObject data) {
        String sql = "INSERT INTO tb_ordercenter_message(usermob,prizeid,roll,ctime,fes) VALUES(?,?,?,SYSDATE(),?)";
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(data.getString("usermob"));
        para.add(data.getInt("prizeid"));
        para.add(data.getString("roll"));
        para.add(data.getString("fes"));
        try {
            return messagedao.insert(sql, para);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 每次随机取出一条相同类型未使用奖品详细信息
     *
     * @param prizeid
     * @return
     * @throws Exception
     */
    public JSONObject queryPrize(int prizeid) throws Exception {
        String sql = "SELECT id,roll,cpid,productid FROM tb_ordercenter_prize WHERE tat = 0 AND prizeid = ? ORDER BY RAND() LIMIT 0,1";
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(prizeid);
        return messagedao.queryPrize(sql, para).get(0);
    }

    /**
     * 修改奖品状态   0未使用 1已发放
     *
     * @param id
     */
    public void updatePrize(int id) {
        String sql = "UPDATE tb_ordercenter_prize SET tat = 1 WHERE id = ? ";
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(id);
        try {
            messagedao.insert(sql, para);
        } catch (Exception e) {
            LOGGER.error("", ReqEnum.ERROR_Exception_db_sql);
            e.printStackTrace();
        }
    }

    /**
     * 修改奖品库存  (奖卷类消耗需修改)
     *
     * @param prizeid
     */
    public void updatePrizeStock(int prizeid) {
        String sql = "UPDATE tb_ordercenter_prizeline SET cou = cou - 1 WHERE id = ? ";
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(prizeid);
        try {
            messagedao.insert(sql, para);
        } catch (Exception e) {
            LOGGER.error("", ReqEnum.ERROR_Exception_db_sql);
            e.printStackTrace();
        }
    }

    /**
     * 查询用户是否已抽取过奖品
     *
     * @param usermob
     * @param fes
     * @return
     * @throws Exception
     */
    public JSONObject queryPrizeMass(String usermob, String fes) {
        String sql =
                "SELECT prizename,li FROM"
                        + " tb_ordercenter_message AS a "
                        + "LEFT JOIN tb_ordercenter_fes_pri AS b ON a.fes = b.fescod AND a.prizeid = b.prizeid "
                        + "LEFT JOIN tb_ordercenter_prizeline AS c ON a.prizeid = c.id AND b.prizeid = c.id "
                        + "WHERE a.fes = ? AND a.usermob = ? ";
        System.out.println(sql);
        ArrayList<Object> para = new ArrayList<Object>();
        para.add(fes);
        para.add(usermob);
        JSONObject mass;
        try {
            mass = messagedao.booleanPrize(sql, para);
            return mass.get("prizename") == null ? null : mass;
        } catch (Exception e) {
            return null;
        }
    }

    public static void main(String[] args) {
        JSONObject object = new JSONObject();
        object.put("fescod", "20050L");

        System.out.println(object.getInt("fescod"));
    }

}
