package com.chinacreator.access.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.bean.FesMessage;
import com.chinacreator.access.bean.RbtMvEntity;
import com.chinacreator.access.dao.RbtMvMapper;
import com.chinacreator.access.task.RbtMvTask;
import com.chinacreator.order.bean.SendBean;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.AESUtil;
import com.chinacreator.order.util.MD5;
import com.chinacreator.order.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 沃音乐视频彩铃试听服务
 * @author jiangtao
 */
@Service
public class RbtMvService {
    private final static Logger LOGGER = LoggerFactory.getLogger(RbtMvService.class);
    private List<RbtMvEntity> cache = new ArrayList<>();
    public static HttpSendService send = new HttpSendServicImp();
    @Autowired
    private RbtMvMapper rbtMvMapper;

    @Value("${direct.rbtMvPwd}")
    private String rbtMvPwd;

    @Value("${direct.rbtMvCpid}")
    private String rbtMvCpid;

    @Value("${direct.rbtMvurl}")
    private String rbtMvurl;

    @Value("${audition.appkey}")
    private String appkey;

    @Value("${audition.appsecret}")
    private String appsecret;

    @Value("${audition.host}")
    private String host;

    @PostConstruct
    public void init() {
        RbtMvTask thread = new RbtMvTask(this);
        new Thread(thread).start();
    }

    public void loadAudition() {
        List<RbtMvEntity> rbtMvEntityList=queryRbtMvList();
        if(rbtMvEntityList!=null){
//            cache = rbtMvMapper.findAll();
            cache=rbtMvEntityList;
        }

    }

    public List<Map<String, String>> findRbtMvs(String productId) {
        List<Map<String, String>> result = new ArrayList<>();

        String timestamp = String.valueOf(System.currentTimeMillis());

        for (RbtMvEntity entity : cache) {
            if (!productId.equals(entity.getProductid())) {
                continue;
            }

            Map<String, String> map = new HashMap<>(16);
            String token = generateTokenId(timestamp, entity.getContentid()).toUpperCase();

            String coverUrl = host + entity.getCover() + "?user=&channelid=" + appkey
                    + "&contentid=" + entity.getContentid() + "&id=" + token
                    + "&timestamp=" + timestamp;

            String videUrl = host + entity.getVideo()+ "?user=&channelid=" + appkey
                    + "&contentid=" + entity.getContentid() + "&id=" + token
                    + "&timestamp=" + timestamp;

            map.put("label", entity.getLabel());
            map.put("contentid", entity.getContentid());
            map.put("contentname", entity.getContentname());
            map.put("coverUrl", coverUrl);
            map.put("videUrl", videUrl);

            result.add(map);
        }

        return result;
    }

    public RbtMvEntity findRbtMv(String contentid) {
        if (StringUtils.isEmpty(contentid)) {
            return null;
        }

        if (cache == null || cache.size() == 0)  {
            loadAudition();
        }

        for (RbtMvEntity entity : cache) {
            if (contentid.equals(entity.getContentid())) {
                return entity;
            }
        }

        return null;
    }

    public String generateTokenId(String timestamp, String contentid) {
        return MD5.MD5Encode(this.appkey + "&SHITING&" + timestamp + "&" + appsecret + "&" + contentid);
    }

    private List<RbtMvEntity> queryRbtMvList(){
        List<RbtMvEntity> rbtMvEntityList=new ArrayList<>();
        JSONObject data=new JSONObject();
        JSONObject jsonObject = new JSONObject();
        JSONObject cpinfo = new JSONObject();
        cpinfo.put("cpid", rbtMvCpid);
        data.put("busiid", "queryRbtMv");
        try {
            cpinfo.put("data", AESUtil.encrypt(data.toString(), rbtMvPwd));
            jsonObject = send.sendPost(rbtMvurl, JSONObject.toJSONString(cpinfo));
            LOGGER.info("订购结果，入参={}, 返回={}", data, jsonObject);
            //下发成功短信
            if ("0000".equals(jsonObject.get("resultCode"))) {
                rbtMvEntityList= JSON.parseArray(jsonObject.getString("data"),RbtMvEntity.class);
            }
        } catch (Exception e) {
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");
            LOGGER.error("订购失败，入参={}, 返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return rbtMvEntityList;
    }
}
