package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.order.util.AESUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 拼图验证码
 */
@Service
public class SliderCaptchaService {

    private static final Logger LOGGER = LoggerFactory.getLogger(SliderCaptchaService.class);

    /**
     * 加密密钥
     */
    private final String KEY = "RIdgvhqzXFotU3On";

    /**
     * 有效期 10s
     */
    private final int DURATION = 1000 * 30;

    /**
     * 生成一个时间的加密串，验证防止用户暴力破解
     * @param datas 用户滑动轨迹
     * @return
     */
    public JSONObject createCaptchaCode(List<Integer> datas) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("resultCode", "0000");
        jsonObject.put("errorInfo", "成功");

        boolean verify = verifyTrails(datas);

        if (!verify) {
            jsonObject.put("resultCode", "1000");
            jsonObject.put("errorInfo", "拼图验证失败");
            return jsonObject;
        }

        try {

            String captchaCode = AESUtil.encrypt(System.currentTimeMillis() + "", KEY);
            jsonObject.put("captchaCode", captchaCode);

        } catch (Exception e) {
            throw new RuntimeException();
        }

        return jsonObject;
    }

    /**
     * 验证用户是否使用了滑动拼图
     * @param captchaCode 拼图轨迹验证通过以后，生成的加密串
     * @return
     */
    public boolean verifyCaptChaCode(String captchaCode) {
        boolean result = true;

        try {

            long createTime = Long.valueOf(AESUtil.decrypt(captchaCode, KEY));
            if (System.currentTimeMillis() - createTime > DURATION) {
                result = false;
            }

        } catch (Exception e) {

            return false;

        } finally {
            LOGGER.info("滑块验证verifyCode，入参={}，返回={}", captchaCode, result);
        }

        return result;
    }

    /**
     * 验证拖动轨迹
     * @param datas
     * @return
     */
    private boolean verifyTrails(List<Integer> datas) {
        // LOGGER.info("==================datas.size=" + datas.size() + "==================");
        int sum = 0;
        for (Integer data : datas) {
            sum += data;
        }
        double avg = sum * 1.0 / datas.size();
        // LOGGER.info("==================avg=" + avg + "==================");

        List<Double> deviations = new ArrayList<>(datas.size());
        for (Integer data : datas) {
            deviations.add(deviations.size(), data - avg);
            // System.out.print("," + (data - avg));
        }

        List<Double> squares = new ArrayList<>(datas.size());
        for (Double data : deviations) {
            squares.add(squares.size(), data * data);
            // System.out.print("," + (data * data));
        }

        double sum2 = 0.0;
        for (Double data : squares) {
            sum2 += data;
        }

        //logger.info("==================sum2=" + sum2 + "==================");
        double stddev = Math.sqrt(sum2 / datas.size());
        //logger.info("==================stddev=" + stddev + "==================");
        return Math.abs(stddev) <= 15 && stddev != 0;
        /*double sum2 = 0.0;
        for (Integer data : datas) {
            sum2 += Math.pow(data - avg, 2);
        }*/

    }
}
