package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 抖音后向叠加包
 */
@Service
public class StackpkgService {

    private static final Logger LOGGER = LoggerFactory.getLogger(StackpkgService.class);

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${outOfStock.key}")
    private String key;

    @Value("${stackpkgPay.url}")
    private String payUrl;

    @Value("${stackpkgQueryOrder.url}")
    private String queryOrderUrl;

    @Value("${stackpkgQueryPay.url}")
    private String queryPayUrl;

    /**
     * 购买
     *
     * @return
     */
    public JSONObject order(JSONObject para, String requestIp) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("usermob", para.getString("usermob"));
        param.put("create_ip", requestIp);
        param.put("attach", para.getString("attach"));
        // "叠加包测试-9元10G", para.getString("productname")
        param.put("title", "抖音后向流量叠加包");
        param.put("orderchannel", para.getString("orderchannel"));
        param.put("sign", SignatureUtil.signatureStr(param, key));
        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(payUrl, param);
            LOGGER.info("叠加包订购结果，入参={}，出参={}", param, resultString);
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.error("叠加包订购失败，入参={}，出参={}", param, e.getMessage());
        }

        JSONObject result = JSONObject.parseObject(resultString);

        return result;
    }

    /**
     * 查询订购关系
     *
     * @param tradeno
     */
    public JSONObject queryOrder(String usermob, String tradeno) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        // param.put("usermob", usermob);
        param.put("out_trade_no", tradeno);
        param.put("sign", SignatureUtil.signatureStr(param, key));
        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(queryOrderUrl, param);
            LOGGER.info("叠加包订购状态查询，参数={}，返回={}", param, resultString);

        } catch (Exception e) {
            LOGGER.error("叠加包订购状态查询失败，参数={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);
        return result;
    }

    /**
     * 查询支付结果
     *
     * @param tradeno
     * @param usermob
     * @Param orderdate  格式：，例子：20190115
     */
    public JSONObject queryPay(String tradeno, String orderdate, String usermob) {
        JSONObject param = new JSONObject();
        param.put("channel", channel);
        param.put("out_trade_no", tradeno);
        param.put("usermob", usermob);
        param.put("order_date", orderdate);
        param.put("sign", SignatureUtil.signatureStr(param, key));

        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(queryPayUrl, param);
            LOGGER.info("返回微信支付结果查询，入参={}，返回={}", param, resultString);

        } catch (Exception e) {
            LOGGER.info("返回微信支付结果查询失败，入参={}，返回={}", param, e.getMessage());
            e.printStackTrace();
        }
        
        param.put("result", resultString);

        JSONObject result = JSONObject.parseObject(resultString);
        return result;
    }
}

