package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.bean.FesMessage;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import com.chinacreator.order.bean.SendBean;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.AESUtil;
import com.chinacreator.order.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

@Service
public class UnicomAppService {

    private static final Logger LOGGER = LoggerFactory.getLogger(UnicomAppService.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");
    public static HttpSendService send = new HttpSendServicImp();

    @Value("${unicomapp.check.ticket}")
    private String url;

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${unicomapp.secret}")
    private String secret;

    @Value("${sendUrl}")
    private String sendUrl;

    @Value("${fValueUrl}")
    private String fValueUrl;

    @Value("${orderInv.url}")
    private String orderInvUrl;

    @Autowired
    private PartService partService;

    @Autowired
    private UsermobService usermobService;

    public JSONObject checkTicket(JSONObject data) throws Exception {
        JSONObject param = new JSONObject();

        // 接口分配
        param.put("channel", channel);
        // 中国联通APP的ticket
        param.put("ticket", data.getString("ticket"));
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        long startTime = System.currentTimeMillis();
        String result = HttpClientUtil.httpClientOfPost(url, param);
        LOGGER.info("调用后台免登录接口：url={},入参={},返回={}", url, param.toJSONString(), result);

        JSONObject jsonObject = JSONObject.parseObject(result);
        if ("0000".equals(jsonObject.get("resultCode"))) {
            String usermob = jsonObject.getString("usermob");
            String cpid = data.getString("cpid");
            jsonObject.put("usermob", usermob);
            jsonObject.put("userid", usermobService.encodeUserId(usermob, cpid));
        }

        long endedTime = System.currentTimeMillis();
        LOGGER.info("中国联通APP免登录，入参={}，返回={}，耗时={}s", param, jsonObject, (endedTime - startTime)/1000);

        return jsonObject;
    }

    public JSONObject order(JSONObject data) {
        JSONObject jsonObject = new JSONObject();

        JSONObject cap = new JSONObject();
        cap.put("cpid", data.getString("cpid"));
        cap.put("productid", data.getString("productid"));
        cap.put("usermob", data.getString("usermob"));

        try {
            SendBean sendBean = null;
            switch (data.getString("productid")) {
                case "2021073001": // 抖音视频彩铃定向流量月包
                    sendBean = Util.retRbtOrderBean(data, 4);
                    break;
                case "2022071807": // 百度定向流量包SVIP会员包29元
                case "2022111408": // 抖音定向流量5G加油包20GB
                case "2022111419": // 抖音定向流量5G加油包5GB
                case "2018072301": // 9元头条 2023-03-15
                case "2022071811": // 30元20G
                    sendBean = Util.retOrderBean(data, 4);
                    break;
                default:
                    break;
            }

            jsonObject = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

            LOGGER.info("订购结果，接口地址={}, 入参={}, 返回={}", sendUrl, sendBean, jsonObject);
            LOGGER_CAP.info("订购结果，入参={}, 返回={}", data, jsonObject);

            //下发成功短信
            if ("0000".equals(jsonObject.get("resultCode"))) {

                int sou = Integer.valueOf(Util._N_to_10(data.getString("sou"), 32));
                String fescod = Util._N_to_10(data.getString("fes"), 32);

                // 用户在前一天打开页面，获取了getstyle，但是在转钟后才提交请求，那么活动结束，需要新的短信
                FesMessage fes = partService.fesExists(fescod, sou);
                if (fes != null) {
                    data.put("new_fescod", fes.getFescod());
                }

                sendBean = Util.retSmsBean(data, 0);
                JSONObject result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

                LOGGER.info("发送订购成功短信，入参={}, 返回={}", data, result);
                LOGGER_CAP.info("发送订购成功短信，入参={}, 返回={}", data, result);
            }
        } catch (Exception e) {
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");
            LOGGER.error("订购失败，入参={}, 返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return jsonObject;
    }

    /**
     * 查询订购、退订状态
     */
    public String queryInfo(JSONObject data) throws Exception {
        String productid = Util.fogdec(data.getString("fog"), "productid");
        String cpid = Util.fogdec(data.getString("fog"), "cpid");

        data.put("productid", productid);
        data.put("cpid", cpid);

        SendBean sendBean = Util.queryInfoBean(data);
        JSONObject result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

        if ("0000".equals(result.getString("resultCode"))) {
            String encryptDsta = result.getString("data");
            String decryptData = AESUtil.decrypt(encryptDsta, Util.keyMap.get(cpid));

            result.put("data", decryptData);
        }

        return result.toString();
    }
}
