package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import com.chinacreator.order.bean.SendBean;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.Util;
import com.github.kevinsawicki.http.HttpRequest;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

@Service
public class WeiboMemberService {

    private static final Logger LOGGER = LoggerFactory.getLogger(WeiboMemberService.class);

    private static HttpSendService send = new HttpSendServicImp();

    //@Value("${sendUrl}")
    //private String sendUrl;

    @Value("${sinaCheckAccount}")
    private String canJoinUrl;

    @Value("${sinaKey}")
    private String key;

    @Value("${secret}")
    private String secret;

    /**
     * 新浪同步订购关系
     */
    @Value("${sinaSyncAccountUrl}")
    private String sinaSyncAccountUrl;

    /**
     * 查询新浪会员的领取状态
     */
    @Value("${sinaQueryAccountUrl}")
    private String sinaQueryAccountUrl;

    @Value("${sendUrl}")
    private String sendUrl;

    @Value("${weibo.queryUserShow}")
    private String weiboUserShowUrl;

    @Autowired
    private UsermobService usermobService;

    /**
     * 领取新浪会员
     * @param data
     * @return
     */
    public JSONObject draw(JSONObject data) throws Exception {
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");
        String channel = Util._N_to_10(data.getString("sou"), 32);

        JSONObject param = new JSONObject();
        // 接口分配
        param.put("channel", "unkc");
        param.put("uid", data.getString("uid"));
        param.put("user_mob", data.getString("usermob"));
        param.put("two_msg", URLDecoder.decode(data.getString("screen_name"), "UTF-8"));
        param.put("cpid", cpid);
        param.put("productid", productid);
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        String result = HttpClientUtil.httpClientOfPost(sinaSyncAccountUrl, param);
        JSONObject jsonObject = JSONObject.parseObject(result);

        LOGGER.info("领取新浪微博会员，入参={}，返回={}", param, result);

        //下发成功短信
        if ("0000".equals(jsonObject.getString("resultCode"))) {
            SendBean sendBean = Util.retSmsBean(data, 211);
            jsonObject = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

            LOGGER.info("领取新浪微博会员成功短信，入参={}，返回={}", data, jsonObject);
        }

        return JSONObject.parseObject(result);
    }


    /**
     * 查询微博用户资料
     * @param data
     * @return
     * @throws Exception
     */
    public JSONObject queryWeiboUserShow(JSONObject data) {
        JSONObject param = new JSONObject();
        param.put("channel","unkc");
        param.put("access_code", data.getString("code"));
        param.put("redirect_uri","http://100.0.4.69:3344/orderchannel/sina/wb/weibo1.html");
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        String result = HttpClientUtil.httpClientOfPost(weiboUserShowUrl, param);
        JSONObject jsonObject = JSONObject.parseObject(result);

        LOGGER.info("新浪微博用户资料查询，入参={}，返回={}", param, jsonObject);

        return jsonObject;
    }

    /**
     * 查询新浪会员的领取状态
     */
    public JSONObject query(JSONObject data) throws Exception {
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");

        JSONObject param = new JSONObject();
        param.put("channel", "unkc"); // 接口分配
        if (data.containsKey("uid") && !StringUtils.isEmpty(data.getString("uid"))) {
            param.put("uid", data.getString("uid"));
        }

        param.put("cpid", cpid);
        param.put("productid", productid);
        param.put("sign", SignatureUtil.signatureStr(param, secret));

        String result = HttpClientUtil.httpClientOfPost(sinaQueryAccountUrl, param);

        JSONObject jsonObject = JSONObject.parseObject(result);

        LOGGER.info("新浪微博会员领取状态查询，入参={}，返回={}", param, jsonObject);

        String usermob = jsonObject.getString("user_mob");
        if (!StringUtils.isEmpty(usermob)) {
            String userid = usermobService.encodeUserId(usermob, cpid);
            jsonObject.put("userid", userid);
            jsonObject.remove("user_mob");
            jsonObject.put("usermob", usermob.substring(0, 3) + "xxxx" + usermob.substring(7, 11));
        }

        String twoMsg = jsonObject.getString("two_msg");
        if (!StringUtils.isEmpty(twoMsg)) {
            jsonObject.put("screen_name", twoMsg);
        }

        return jsonObject;
    }

    /**
     * 是否有领取会员的资格
     * @param uid 微博账号uid
     * @return
     */
    public String canJoin(String uid) {
        //String url = "https://new.vip.weibo.cn/unicom/checkaccount";
        //String uid = "3542915174";
        //String key = "ldEfdLxPV5md6GFQ6vzEPlL2w6g22Tuf";

        String timestamp = String.valueOf(System.currentTimeMillis());
        String sign = DigestUtils.md5Hex(uid + timestamp + key);

        Map<String, String> params = new HashMap<>();
        params.put("uid", uid);
        params.put("timestamp", timestamp);
        params.put("sign", sign);

        HttpRequest request = HttpRequest.post(canJoinUrl).form(params);
        String result = new String(request.bytes(), Charset.forName("UTF-8"));

        result = result.replaceAll("resultInfo", "errorInfo");

        LOGGER.info("新浪微博会员领取资格校验，入参={}，返回={}", params, result);

        return result;
    }

    public static void main(String[] args) throws UnsupportedEncodingException {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("as", null);
        System.out.println(URLDecoder.decode("%E7%94%A8%E6%88%B7", "UTF-8"));
    }
}
