package com.chinacreator.access.service;

import com.chinacreator.access.bean.FesMessage;
import com.chinacreator.access.dao.OrderMapper;
import com.chinacreator.access.util.DateUtil;
import com.chinacreator.access.util.SerialnumberUtil;
import com.chinacreator.order.bean.DirectedJyLogBean;
import com.chinacreator.order.bean.DirectedJyOrderBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import com.chinacreator.order.bean.SendBean;
import com.chinacreator.order.send.HttpSendService;
import com.chinacreator.order.send.imp.HttpSendServicImp;
import com.chinacreator.order.util.Util;

/**
 * @author jiangtao
 */
@Service
public class WeiboOrderService {

    private final static Logger LOGGER = LoggerFactory.getLogger(WeiboOrderService.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");

    public static HttpSendService send = new HttpSendServicImp();

    @Value("${sendUrl}")
    private String sendUrl;

    @Value("${fValueUrl}")
    private String fValueUrl;

    @Value("${secret}")
    private String secret;

    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private PartService partService;

    public JSONObject order(JSONObject data) {
        JSONObject jsonObject = null;

        JSONObject cap = new JSONObject();
        cap.put("cpid", data.getString("cpid"));
        cap.put("productid", data.getString("productid"));
        cap.put("usermob", data.getString("usermob"));

        try {
            SendBean sendBean = Util.retOrderBean(data, 0);
            jsonObject = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

            LOGGER.info("订购结果，入参={}, 返回={}", data, jsonObject);

            //下发成功短信
            if ("0000".equals(jsonObject.get("resultCode"))) {
                try {
                    DirectedJyOrderBean directedJyOrderBean = new DirectedJyOrderBean();
                    String orderid = SerialnumberUtil.Getnum(10000000,99999999);
                    directedJyOrderBean.setOrderid(orderid);
                    directedJyOrderBean.setCpid(cap.getString("cpid"));
                    directedJyOrderBean.setBusiid("unOrder");
                    directedJyOrderBean.setUsermob(cap.getString("usermob"));
                    directedJyOrderBean.setProductid(cap.getString("productid"));
                    directedJyOrderBean.setOrderchannel(data.getString("orderchannel"));
                    directedJyOrderBean.setCurstatus("0");
                    directedJyOrderBean.setOrdertime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));
                    orderMapper.insertDirectedJyOrder(directedJyOrderBean);
                }catch (Exception er){
                    er.printStackTrace();
                }

                int sou = Integer.valueOf(Util._N_to_10(data.getString("sou"), 32));
                String fescod = Util._N_to_10(data.getString("fes"), 32);

                // 用户在前一天打开页面，获取了getstyle，但是在转钟后才提交请求，那么活动结束，需要新的短信
                FesMessage fes = partService.fesExists(fescod, sou);
                if (fes != null) {
                    data.put("new_fescod", fes.getFescod());
                }

                sendBean = Util.retSmsBean(data, 0);
                JSONObject result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

                LOGGER.info("发送订购成功短信，入参={}, 返回={}", data, result);
                LOGGER_CAP.info("发送订购成功短信，入参={}, 返回={}", cap, result);
            }

            //是否为空都需要调用？保存新浪F值
            saveValueF(data);

            //记录日志表
            try {
                DirectedJyLogBean logBean = new DirectedJyLogBean();
                logBean.setOrderid(SerialnumberUtil.Getnum(10000000,99999999));
                logBean.setCpid(data.getString("cpid"));
                logBean.setBusiid("cancel");
                logBean.setUsermob(data.getString("usermob"));
                logBean.setProductid(data.getString("productid"));
                logBean.setOrderchannel(data.getString("orderchannel"));
                logBean.setRequestparam(JSONObject.toJSONString(sendBean));
                logBean.setResponsecode(jsonObject.get("resultCode")==null?"":jsonObject.get("resultCode").toString());
                logBean.setResponseinfo(jsonObject.get("errorInfo")==null?"":jsonObject.get("errorInfo").toString());
                logBean.setRequesttime(DateUtil.now("yyyy-MM-dd HH:mm:ss.SSS"));

                orderMapper.insertDirectedJyLog(logBean);
            }catch (Exception ef){
                ef.printStackTrace();
            }

        } catch (Exception e) {
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");
            LOGGER.error("订购失败，入参={}, 返回={}", data, e.getMessage());
            e.printStackTrace();
        }

        return jsonObject;
    }

    /**
     * 新浪微博的F值，标记从新浪客户端的那个入口订购的
     * @param data
     * @return
     */
    public JSONObject saveValueF(JSONObject data) {
        String cpid = Util.fogdec(data.getString("fog"), "cpid");
        String productid = Util.fogdec(data.getString("fog"), "productid");

        JSONObject params = new JSONObject();
        try {
            params.put("uid", data.getString("useraccount"));
            params.put("channel", "unkc");
            params.put("user_mob", data.getString("usermob"));
            params.put("one_msg", data.getString("f"));
            params.put("cpid", cpid);
            params.put("productid", productid);
            String sign = SignatureUtil.signatureStr(params, secret);
            params.put("sign", sign);

            String ret = HttpClientUtil.httpClientOfPost(fValueUrl, params);
            LOGGER.info("保存F值，入参={}，返回={}", params, ret);

            return JSONObject.parseObject(ret);

        } catch (Exception e) {
            LOGGER.error("保存F值失败，入参={}，返回={}", params.toString(), e.getMessage());
            e.printStackTrace();
        }

        return null;
    }

    /*public JSONObject cancel(JSONObject data) {
        JSONObject jsonObject = null;

        try {
            SendBean sendBean = Util.retOrderBean(data, 1);
            jsonObject = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

            if ("0000".equals(jsonObject.get("resultCode"))) {//成功
                //下发成功短信
                sendBean = Util.retSmsBean(data, 1);
                send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("退订失败：{}", data.toString());
            jsonObject = new JSONObject();

            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");
        }

        return jsonObject;
    }


     * 查询订购状态

    public String queryOrder(JSONObject data) throws Exception {
        String productid = Util.fogdec(data.getString("fog"), "productid");
        String cpid = Util.fogdec(data.getString("fog"), "cpid");

        data.put("productid", productid);
        data.put("cpid", cpid);

        SendBean sendBean = Util.queryOrderBean(data);
        JSONObject result = send.sendPost(sendUrl, JSONObject.toJSONString(sendBean));

        if ("0000".equals(result.getString("resultCode"))) {
            String encryptDsta = result.getString("data");
            String decryptData = AESUtil.decrypt(encryptDsta, Util.keyMap.get(cpid));

            result.put("data", decryptData);
        }

        return result.toString();
    }*/
}
