package com.chinacreator.access.service;

import com.alibaba.fastjson.JSONObject;
import com.chinacreator.access.util.HttpClientUtil;
import com.chinacreator.access.util.SignatureUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

/**
 * 音乐渠道订购抖音后向流量包
 */
@Service
public class WomusicService {

    private static final Logger LOGGER = LoggerFactory.getLogger(WomusicService.class);

    @Value("${outOfStock.url}")
    private String url;

    @Value("${outOfStock.channel}")
    private String channel;

    @Value("${outOfStock.key}")
    private String key;

    @Value("${queryMusic.url}")
    private String queryWoMusicUrl;

    @Value("${resreatMusic.url}")
    private String resreatMusicUrl;

    public JSONObject queryStock(JSONObject para) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("channel", channel);
        jsonObject.put("stock_code", para.get("stockcode"));
        jsonObject.put("oper_ip", para.get("operip"));
        jsonObject.put("dic_cpid", "music_channel");

        jsonObject.put("sign", SignatureUtil.signatureStr(jsonObject, key));
        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(queryWoMusicUrl, jsonObject);

            LOGGER.info("沃音乐查询库存，入参={}，返回={}", jsonObject, resultString);
        } catch (Exception e) {
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");

            LOGGER.error("沃音乐查询库存失败，入参={}，返回={}", jsonObject, e.getMessage());
            e.printStackTrace();

        }

        JSONObject result = JSONObject.parseObject(resultString);

        return result;
    }

    public JSONObject order(JSONObject para) {
        JSONObject jsonObject = new JSONObject();

        jsonObject.put("channel", channel);
        jsonObject.put("user_mob", para.get("usermob"));
        jsonObject.put("stock_code", para.get("stockcode"));
        jsonObject.put("oper_ip", para.get("operip"));
        jsonObject.put("dic_cpid", "music_channel");

        jsonObject.put("sign", SignatureUtil.signatureStr(jsonObject, key));

        String resultString = "";

        try {
            resultString = HttpClientUtil.httpClientOfPost(url, jsonObject);

            LOGGER.info("沃音乐订购，入参={}，返回={}", jsonObject, resultString);
        } catch (Exception e) {
            e.printStackTrace();
            jsonObject.put("resultCode", "8000");
            jsonObject.put("errorInfo", "系统忙，请稍候再试");

            LOGGER.info("沃音乐订购失败：入参={}，返回={}", jsonObject, e.getMessage());
        }

        JSONObject result = JSONObject.parseObject(resultString);

        return result;
    }

    /**
     * 音乐退款
     */
    public JSONObject refund(String stockCode, String operIp) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("channel", channel);
        jsonObject.put("stock_code", stockCode);
        jsonObject.put("oper_ip", operIp);
        jsonObject.put("dic_cpid", "music_channel");
        jsonObject.put("sign", SignatureUtil.signatureStr(jsonObject, key));
        String resultString = "";

        try {

            resultString = HttpClientUtil.httpClientOfPost(resreatMusicUrl, jsonObject);

        } catch (Exception e) {
            e.printStackTrace();
        }

        JSONObject result = JSONObject.parseObject(resultString);

        return result;
    }
}
