package com.chinacreator.access.servlet;

import com.chinacreator.access.bean.ChannelInfo;
import com.chinacreator.access.restrain.ReqEnum;
import com.chinacreator.access.service.ChannelInfoService;
import com.chinacreator.access.service.DxService;
import com.chinacreator.access.service.UsermobService;
import com.chinacreator.access.util.TransUtil;
import com.chinacreator.order.util.Util;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.net.URLDecoder;

/**
 * 业务逻辑上会与定向接口产生交互
 * 订购,退订,流量查询,订购关系查询
 *
 * @author Yk
 */
@RestController
@RequestMapping("/com")
public class ComboController {

    private static final Logger LOGGER = LoggerFactory.getLogger(ComboController.class);
    private static final Logger LOGGER_CAP = LoggerFactory.getLogger("LOGGER_CAP");

    @Autowired
    private DxService dxService;

    @Autowired
    private UsermobService usermobService;

    @Autowired
    private ChannelInfoService channelInfoService;

    /**
     * 订购统一认证接口
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/buyc", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void buyc(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToJSON(request);
        JSONObject dataCap = new JSONObject();
        JSONObject result = null;
        try {
            // 如果是伪码
            if (data.containsKey("userid")) {
                String userid = data.getString("userid");
                String cpid = Util.fogdec(data.getString("fog"), "cpid");
                data.put("usermob", usermobService.decodeUserId(userid, cpid));
            }
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();
            data.put("smschannel", smschannel);

            LOGGER.info("前向订购，入参={}", data);

            String productid = Util.fogdec(data.getString("fog"), "productid");
            dataCap.put("productid", productid);
            dataCap.put("channel", data.getString("sou"));
            dataCap.put("token", data.getString("token"));
            LOGGER_CAP.info("产品订购，入参={}", dataCap);

            result = dxService.submitSc(data, 0);
            if ("6631".equals(result.getString("resultCode"))) {
                result.put("errorInfo", "用户已欠费，请缴费后重新发起！");
            }
            TransUtil.resToJSON(response, result);
        } catch (Exception e) {
            LOGGER.error("前向订购，入参={}，返回={}", data, e.getMessage());
            LOGGER_CAP.info("产品订购，入参={}，返回={}", dataCap, e.getMessage());

            data = Util.reqJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
            TransUtil.resToJSON(response, data);
        }

        LOGGER_CAP.info("产品订购，入参={}，返回={}", dataCap, result);
    }

    /**
     * 退订接口
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/unsu", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void unsu(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToJSON(request);
        try {
            String orderchannel = Util._N_to_10(data.getString("sou"), 32);
            ChannelInfo channelInfo = channelInfoService.findChannelInfo(orderchannel);
            String smschannel = channelInfo == null ? "" : channelInfo.getSmschannel();
            data.put("smschannel", smschannel);

            LOGGER.info("前向退订(unsu)，入参={}", data);

            TransUtil.resToJSON(response, dxService.submitSc(data, 1));
        } catch (Exception e) {
            LOGGER.error("前向退订(unsu)，入参={}，返回={}", data, e.getMessage());

            data = Util.reqJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
            TransUtil.resToJSON(response, data);
        }
    }


    /**
     * 退订接口查询号码退订当前cpid下产品
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/unsuc", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void unsuc(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToJSON(request);
        try {
            String cpid = Util.fogdec(data.getString("fog"), "cpid");
            String productid = Util.fogdec(data.getString("fog"), "productid");
            data.put("cpid", cpid);
            data.put("productid", productid);

            if (!StringUtils.isEmpty(data.getString("canceltype")) &&
                    !"undefined".equals(data.getString("canceltype"))) {
                data.put("canceltype", URLDecoder.decode(data.getString("canceltype"), "UTF-8"));
            } else {
                data.put("canceltype", "");
            }

            if (!StringUtils.isEmpty(data.getString("cancelmsg")) &&
                    !"undefined".equals(data.getString("cancelmsg"))) {
                data.put("cancelmsg", URLDecoder.decode(data.getString("cancelmsg"), "UTF-8"));
            } else {
                data.put("cancelmsg", "");
            }

            LOGGER.info("前向退订，入参={}", data);

            switch (productid) {
                case "2018072301":
                case "2018122101":
                    TransUtil.resToJSON(response, dxService.cancelSc(data, 1));
                    break;
                default:

                    TransUtil.resToJSON(response, Util.reqJsonErr(ReqEnum.ERROR_Exception_encrypt_decrypt));
                    break;
            }
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.error("前向退订(unsuc)，入参={}，返回={}", data, e.getMessage());
            TransUtil.resToJSON(response, Util.reqJsonErr(ReqEnum.ERROR_Exception_system_decrypt));
        }
    }

    /**
     * 激活
     * 此类接口只用于需要同步的头条系接口
     * 退订接口根据号码退订当前cpid下产品(对有多种产品用户的退订)
     *
     * @param request  activateSc
     * @param response
     */
    @RequestMapping(value = "/act", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void act(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToJSON(request);
        try {
            LOGGER.info("前向激活，入参={}", data);
            JSONObject jsonObject = dxService.activateSc(data, 2);
            TransUtil.resToJSON(response, jsonObject);
            LOGGER.info("前向激活，入参={}，返回={}", data, jsonObject);
        } catch (Exception e) {
            LOGGER.error("前向激活失败，入参={}，返回={}", data, e.getMessage());
            Util.reqJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
        }
    }

    /**
     * 预约接口
     * 退订接口感觉号码退订当前cpid下产品(对有多种产品用户的退订)
     *
     * @param request  activateSc
     * @param response
     */
    @RequestMapping(value = "/yuyue", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void yuyue(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToJSON(request);
        try {
            LOGGER.info("前向预约订购(yuyue)，入参={}", data);
            TransUtil.resToJSON(response, dxService.yuyueSc(data));
        } catch (Exception e) {
            LOGGER.error("前向预约订购失败(yuyue)，入参={}，返回={}", data, e.getMessage());
            Util.reqJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
        }
    }

    /**
     * 根据用户设备号获取头条端存储的手机号码
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/dev", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void dev(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToJSON(request);
        try {
            LOGGER.info("头条端用设备号取手机号码，入参={}", data);
            TransUtil.resToJSON(response, dxService.devToPhone(data));
        } catch (Exception e) {
            LOGGER.error("头条端用设备号取手机号码，入参={}，返回={}", data, e.getMessage());
            Util.reqJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
        }
    }

    /**
     * 根据客户cpid与用户手机号查询流量
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/queryl", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void queryl(HttpServletRequest request, HttpServletResponse response) {
        JSONObject data = TransUtil.reqToJSON(request);
        try {
            LOGGER.info("查询流量，入参={}", data);
            TransUtil.resToJSON(response, dxService.cpidAndPhoneToFolw(data));
        } catch (Exception e) {
            LOGGER.error("查询流量，入参={}，返回={}", data, e.getMessage());
            Util.reqJsonErr(ReqEnum.ERROR_Exception_system_decrypt);
        }
    }


    /**
     * 已经不再使用了
     * @param request
     * @param response
     */
    @RequestMapping(value = "/sms", method = RequestMethod.POST, produces = "application/json;charset=UTF-8")
    public void sendSms(HttpServletRequest request, HttpServletResponse response) {

    }

}
