package com.chinacreator.access.util;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

public class IpAddrUtil {

    public static String getIpAddr(HttpServletRequest request) {
        String ip = request.getHeader("X-Real-IP");
        if (inValidateIpAddr(ip)) {
            ip = request.getHeader("X-Forwarded-For");
        }
        if (inValidateIpAddr(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (inValidateIpAddr(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (inValidateIpAddr(ip)) {
            ip = request.getHeader("clientip");
        }
        if (inValidateIpAddr(ip)) {
            ip = request.getRemoteAddr();
        }
        if (!inValidateIpAddr(ip)) {
            int index = ip.indexOf(",");
            if (index != -1) {
                ip = ip.substring(0, index);
            }
        }
        return ip;
    }

    /**
     * ip地址是否有效
     *
     * @param ip
     * @return true：无效，false有效
     */
    private static boolean inValidateIpAddr(String ip) {
        return ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)
                || ip.startsWith("10.") || ip.startsWith("100.")
                || ip.startsWith("172.") || ip.startsWith("192.")
                || ip.startsWith("114.255.201.224");
    }

    public static String getUserAgent(HttpServletRequest request) {
        return request.getHeader("User-Agent");
    }

    public static String getHostIp() {
        try {
            Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces();
            while (en.hasMoreElements()) {
                NetworkInterface intf = en.nextElement();
                Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses();
                while (enumIpAddr.hasMoreElements()) {
                    InetAddress inetAddress = enumIpAddr.nextElement();
                    if (!inetAddress.isLoopbackAddress() && !inetAddress.isLinkLocalAddress() && inetAddress.isSiteLocalAddress()) {
                        return inetAddress.getHostAddress();
                    }
                }
            }
        } catch (SocketException e) {
            e.printStackTrace();
        }
        return null;
    }

    public static boolean isBlockIP(String blockIP, String remoteIp) {
        return blockIP.indexOf(remoteIp) >= 0;
    }
}
