package com.chinacreator.access.util;

import net.sf.json.JSONObject;
import org.apache.commons.ssl.SSLClient;

import javax.net.ssl.HttpsURLConnection;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;

/**
 * @author jiangtao
 */
public class TransUtil {

    // private static final Logger LOGGER = LoggerFactory.getLogger(TransUtil.class);

    /**
     * 将获取的请求转Json
     *
     * @param request 请求
     * @return 请求来的参数转换后的Json数据
     */
    public static com.alibaba.fastjson.JSONObject reqToAliJSON(HttpServletRequest request) {
        StringBuffer reqjson = new StringBuffer();
        String line = null;
        try {
            BufferedReader reader = request.getReader();
            while ((line = reader.readLine()) != null) {
                reqjson.append(line);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        com.alibaba.fastjson.JSONObject data = com.alibaba.fastjson.JSONObject.parseObject(reqjson.toString());
        // LOGGER.info("获取数据:" + data.toString());
        return data;
    }

    /**
     * 将获取的请求转Json
     *
     * @param request 请求
     * @return 请求来的参数转换后的Json数据，建议使用reqToAliJSON
     */
    @Deprecated
    public static JSONObject reqToJSON(HttpServletRequest request) {
        StringBuffer reqjson = new StringBuffer();
        String line = null;
        try {
            BufferedReader reader = request.getReader();
            while ((line = reader.readLine()) != null) {
                reqjson.append(line);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        JSONObject data = JSONObject.fromObject(reqjson.toString());
        // LOGGER.info("获取数据:" + data.toString());
        return data;
    }

    /**
     * 将返回结果转Json返回
     *
     * @param response 返回对象
     * @param data     结果对象
     */
    public static void resToJSON(HttpServletResponse response, JSONObject data) {
        OutputStream out;
        try {
            out = response.getOutputStream();
            out.write(data.toString().getBytes(StandardCharsets.UTF_8));
            out.flush();
            // LOGGER.info("返回结果：" + data.toString());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 解读流
     *
     * @param inStream
     * @return
     * @throws Exception
     */
    public static String readInputStream(InputStream inStream) throws Exception {
        ByteArrayOutputStream outStream = null;
        try {
            outStream = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int len = 0;
            while ((len = inStream.read(buffer)) != -1) {
                outStream.write(buffer, 0, len);
            }
            byte[] data = outStream.toByteArray();
            return new String(data, StandardCharsets.UTF_8);
        } finally {
            if (outStream != null) {
                outStream.close();
            }
            if (inStream != null) {
                inStream.close();
            }
        }
    }

    /**
     * 以get方式获取链接的输出
     *
     * @param href
     * @return
     * @throws Exception
     */
    public static String get(String href) throws Exception {
        URL url = new URL(href);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.addRequestProperty("content-type", "text/plain");
        conn.setUseCaches(false);
        conn.setConnectTimeout(30000);
        conn.setDoOutput(false);
        return readInputStream(conn.getInputStream());
    }

    /**
     * 以post方式获取链接的输出
     *
     * @param href
     * @return
     * @throws Exception
     */
    public static String post(String href, String data) throws Exception {
        HttpURLConnection conn = null;
        OutputStream os = null;
        if (data == null || data.length() == 0) {
            return get(href);
        } else {
            try {
                URL url = new URL(href);
                conn = (HttpURLConnection) url.openConnection();
                if (conn == null) {
                    throw new IllegalArgumentException("The conn cannot be null");
                }
                conn.setRequestMethod("POST");
                conn.addRequestProperty("content-type", "text/plain");
                conn.setDoOutput(true);
                conn.setUseCaches(false);
                os = conn.getOutputStream();
                if (os == null) {
                    throw new IllegalArgumentException("The os cannot be null");
                }
                os.write(data.getBytes(StandardCharsets.UTF_8));
                os.flush();
            } finally {
                if (conn != null) {
                    conn.getOutputStream().close();
                }
            }
            if (conn != null) {
                return readInputStream(conn.getInputStream());
            }
            return null;
        }
    }

    public static String post(String reqUrl, String data, SSLClient client) throws Exception {
        URL url = new URL(reqUrl);
        HttpsURLConnection httpUrlConn = (HttpsURLConnection) url.openConnection();
        httpUrlConn.setSSLSocketFactory(client);
        /* 发送数据 */
        httpUrlConn.setDoOutput(true);
        httpUrlConn.setDoInput(true);
        httpUrlConn.setRequestMethod("POST");
        httpUrlConn.setRequestProperty("Accept", "application/json");
        httpUrlConn.setRequestProperty("Content-Type", "application/json");
        httpUrlConn.connect();
        OutputStreamWriter out = new OutputStreamWriter(httpUrlConn.getOutputStream(), "UTF-8");
        out.append(data);
        out.flush();
        out.close();

        /* 接收数据 */
        String result = "";
        //服务器返回东西了，先对响应码判断
        if (httpUrlConn.getResponseCode() == 200) {
            result = readInputStream(httpUrlConn.getInputStream());
        } else {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("resultCode", "8001");
            jsonObject.put("errorInfo", "业务调用失败！" + httpUrlConn.getResponseCode() + "," + httpUrlConn.getResponseMessage());
            result = jsonObject.toString();
        }
        httpUrlConn.disconnect();
        return result;
    }

    /**
     * 以post方式获取链接的输出
     *
     * @param href
     * @return
     * @throws Exception
     */
    public static String postc(String href, String data) throws Exception {
        HttpURLConnection conn = null;
        if (data == null || data.length() == 0) {
            return get(href);
        } else {
            try {
                URL url = new URL(href);
                conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("POST");
                conn.addRequestProperty("content-type", "text/plain");
                conn.setDoOutput(true);
                conn.setUseCaches(false);
                OutputStream os = conn.getOutputStream();
                os.write(data.getBytes(StandardCharsets.UTF_8));
                os.flush();
            } finally {
                if (conn != null) {
                    conn.getOutputStream().close();
                }
            }
            if (conn != null) {
                return readInputStream(conn.getInputStream());
            }
            return null;
        }
    }
}
