package com.chinacreator.ecd.wechat;

import com.alibaba.fastjson.JSON;
import com.chinacreator.ecd.wechat.modal.JssdkTicket;
import com.chinacreator.ecd.wechat.service.JsSdkService;
import com.chinacreator.ecd.wechat.service.WechatConfig;
import com.chinacreator.ecd.wechat.util.StringUtil;
import com.github.kevinsawicki.http.HttpRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/jssdk")
public class JssdkController {

    private static final Integer RANDOM_LENGTH = 12;

    @Autowired
    private JsSdkService jsSdkService;

    @Value("${gtoken.url}")
    private String gtokenUrl;

    /**
     * 获取服务器的真实外网地址，因为服务的代理很多，不知道真正的出口在哪里
     * 通过腾讯返回的错误，可知道真实的外网地址
     *
     * @return
     */
    @RequestMapping("/gtoken")
    public String token() {
        // String url = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid={0}&secret={1}";
        String url = gtokenUrl + "?grant_type=client_credential&appid={0}&secret={1}";

        String reqUrl = StringUtil.parse(url, WechatConfig.appid, WechatConfig.secret);

        System.out.println("请求地址是：" + reqUrl);

        HttpRequest request = HttpRequest.get(reqUrl);

        //String result = new String(request.bytes(), Charset.forName("UTF-8"));
        //System.out.println("获取global_access_token的结果是：" + result);

        return new String(request.bytes(), StandardCharsets.UTF_8);
    }

    /**
     * 在使用微信的JSSDK前需获取网页的配置信息
     *
     * @param request
     * @return
     */
    @RequestMapping("/init")
    public String init(HttpServletRequest request) {
        String nonceStr = StringUtil.getRandomString(RANDOM_LENGTH);
        String jssdkTicket = JSON.parseObject(jsSdkService.obtainJsapiTicket(), JssdkTicket.class).getTicket();
        float roun = System.currentTimeMillis() / 1000;//Math.round 需使用float
        Integer timestamp = Math.round(roun);
        System.out.println("获取到的url的参数是：nonceStr" + nonceStr + ",jssdkTicket:" + jssdkTicket + ",timestamp: " + timestamp);
        String url = request.getParameter("url");
        if (StringUtils.isEmpty(url)) {
            return "{\"errmsg\":\"url参数不能为空\"}";
        }
        Map<String, String> params = new HashMap<String, String>();
        params.put("appId", WechatConfig.appid);
        params.put("nonceStr", nonceStr);
        params.put("jssdkTicket", jssdkTicket);
        params.put("timestamp", timestamp.toString());
        params.put("link", url);
        System.out.println(url);
        String signature = "";
        try {
            signature = JsSdkService.sign(nonceStr, jssdkTicket, timestamp, url);
        } catch (Exception e) {
            return JSON.toJSONString(params);
        }
        params.put("signature", signature);
        return JSON.toJSONString(params);
    }
}
