package com.chinacreator.order.dao.imp;

import com.chinacreator.order.bean.DirectedJyLogBean;
import com.chinacreator.order.bean.DirectedJyOrderBean;
import com.chinacreator.order.dao.QueryDao;
import com.chinacreator.order.util.JDBCUtils;
import com.chinacreator.order.util.Util;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.*;
import java.util.List;

public class QueryDaoImp implements QueryDao {
    Logger logger = LoggerFactory.getLogger(getClass());

    @Override
    public String querykeyWord(String cpid) throws ClassNotFoundException, SQLException {
        String sql = "SELECT keypwd FROM tb_directed_cpinfo "
                + "WHERE cpid = ?";
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        logger.info("链接物理地址-:" + con.hashCode());
        String keyWord = null;
        try {
            stmt = con.prepareStatement(sql);
            stmt.setString(1, cpid);
            rs = stmt.executeQuery();
            while (rs.next()) {
                keyWord = rs.getString(1);
                logger.info(cpid + "----秘钥 ----" + keyWord + "\t");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return keyWord;
    }

    @Override //插入发送成后的验证码
    public boolean insertCode(String cpid, String productid, String code,
                              String usermobs, String smstype, Integer sou) throws Exception {
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        String sql = "INSERT INTO tb_ordercenter_code(cpid,productid,code,usermob,btime,smstype,sou) VALUES (?,?,?,?,?,?,?)";
        stmt = con.prepareStatement(sql);
        stmt.setString(1, cpid);
        stmt.setString(2, productid);
        stmt.setString(3, code);
        stmt.setString(4, usermobs);
        stmt.setTimestamp(5, new Timestamp(System.currentTimeMillis()));
        stmt.setString(6, smstype);
        stmt.setInt(7, sou);
        try {
            int Res = stmt.executeUpdate();
            relo = Res > 0;

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }


    @Override//更新验证码对比时间
    public boolean  updateCode(String cpid, String productid, String ncode,
                              String usermobs) throws Exception {
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        String sql =
                "UPDATE tb_ordercenter_code " +
                        "SET ctime = ? " +
                        "WHERE " +
                        "cpid = ? AND " +
                        "productid = ? AND " +
                        "code = ? AND " +
                        "usermob = ? AND " +
                        "btime > ? AND ISNULL(ctime)";
        stmt = con.prepareStatement(sql);
        stmt.setTimestamp(1, new Timestamp(System.currentTimeMillis()));
        stmt.setString(2, cpid);
        stmt.setString(3, productid);
        stmt.setString(4, ncode);
        stmt.setString(5, usermobs);
        stmt.setTimestamp(6, Util.beforeDate(5));
        try {
            int Res = stmt.executeUpdate();
            relo = Res > 0;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }

    @Override//插入验证码记录表
    public boolean insertCodeLim(String usermobs, String cpid, String smstype) throws Exception {
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        String sql = "INSERT INTO tb_ordercenter_codelim(usermob,cpid,smstype,lim) VALUES (?,?,?,?)";
        stmt = con.prepareStatement(sql);
        stmt.setString(1, usermobs);
        stmt.setString(2, cpid);
        stmt.setString(3, smstype);
        stmt.setInt(4, 1);
        try {
            int Res = stmt.executeUpdate();
            relo = Res > 0;

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }

    @Override//更新验证码记录表
    public boolean updatecodeLim(String usermobs, String cpid, String smstype) throws Exception {
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        String sql =
                "UPDATE tb_ordercenter_codelim " +
                        "SET lim = lim+1 " +
                        "WHERE " +
                        "usermob = ? AND " +
                        "cpid = ? AND " +
                        "smstype = ?";
        stmt = con.prepareStatement(sql);
        stmt.setString(1, usermobs);
        stmt.setString(2, cpid);
        stmt.setString(3, smstype);
        try {
            int Res = stmt.executeUpdate();
            relo = Res > 0;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }

    @Override//判断手机号是否今日发送过短信与是否达到上限
    public int intcodeLim(String usermobs, String cpid, String smstype) throws Exception {
        String sql = "SELECT lim FROM tb_ordercenter_codelim "
                + "WHERE usermob = ? AND cpid = ? AND smstype = ?";
        int lim = 0;
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        try {
            stmt = con.prepareStatement(sql);
            stmt.setString(1, usermobs);
            stmt.setString(2, cpid);
            stmt.setString(3, smstype);
            rs = stmt.executeQuery();
            while (rs.next()) {
                lim = rs.getInt(1);
                logger.info("----lim ----" + lim + "\t");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return lim;
    }


    @Override
    public String querySms(String fes, String cpid, String productid, int i) throws Exception {
        String sql = "";
        if (i == 0) {
            sql = "SELECT smsord FROM tb_ordercenter_product WHERE cpid = ? AND productid = ? AND fescod = ?";
        }
        if (i == 1) {
            sql = "SELECT smscan FROM tb_ordercenter_product WHERE cpid = ? AND productid = ? AND fescod = ?";
        }
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        logger.info("链接物理地址-:" + con.hashCode());
        PreparedStatement stmt = null;
        ResultSet rs = null;
        String keyWord = null;
        try {
            stmt = con.prepareStatement(sql);
            stmt.setString(1, cpid);
            stmt.setString(2, productid);
            stmt.setString(3, fes);
            rs = stmt.executeQuery();
            while (rs.next()) {
                keyWord = rs.getString(1);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return keyWord;
    }

    /**
     * 取出号码发送验证码中时间最大时间。
     */
    @Override
    public boolean intcodeTim(String usermobs, String cpid, String smstype)
            throws Exception {
        String sql =
                "SELECT COUNT(1) AS co FROM ("
                        + " SELECT IFNULL(MAX(btime),'2016-01-01 00:00:00') AS ti FROM tb_ordercenter_code"
                        + " WHERE usermob = ?"
                        + " AND cpid = ?"
                        + " AND smstype = ? "
                        + ") AS T WHERE  ti < ?";
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean ret = false;
        try {
            stmt = con.prepareStatement(sql);
            stmt.setString(1, usermobs);
            stmt.setString(2, cpid);
            stmt.setString(3, smstype);
            stmt.setTimestamp(4, Util.beforeDate(1));
            rs = stmt.executeQuery();
            while (rs.next()) {
                ret = !rs.getString(1).equals("0");
            }
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return ret;
    }

    /**
     * alter table tb_ordercenter_code add tries int default 0 comment '尝试次数，最多5次';
     * @param cpid
     * @param productid
     * @param usermob
     * @return
     */
    @Override
    public boolean checkTries(String cpid, String productid, String usermob) throws SQLException {
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        String sql =
            "UPDATE tb_ordercenter_code " +
                "SET tries = tries+1 " +
                "WHERE " +
                " cpid = ? " +
                "AND productid = ? " +
                "AND usermob = ? " +
                "and tries <= 4 " +
                "and btime > ? AND ISNULL(ctime)";
        stmt = con.prepareStatement(sql);
        stmt.setString(1, cpid);
        stmt.setString(2, productid);
        stmt.setString(3, usermob);
        stmt.setTimestamp(4, Util.beforeDate(5));
        try {
            int Res = stmt.executeUpdate();
            relo = Res > 0;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }

    @Override
    public boolean insertDirectedJyBean(DirectedJyOrderBean bean) throws Exception {
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        String sql = "INSERT into td_directed_order(orderid,cpid,productid,productnewid,usermob, " +
                "usertype,subscrbtype,curstatus,ordertime,orderchannel,canceltime,cancelchannel,province,city,producttype,useraccount,directedtype,note) " +
                " VALUES(?, ?, ?, ?, ? " +
                " , ?, ?, ?, ?, ?, ?, ?, ?, " +
                "  ?, ?, ?, ?, ?)";
        stmt = con.prepareStatement(sql);
        stmt.setString(1, bean.getOrderid());
        stmt.setString(2, bean.getCpid());
        stmt.setString(3, bean.getProductid());
        stmt.setString(4, bean.getProductnewid());
        stmt.setString(5, bean.getUsermob());
        stmt.setString(6, bean.getUsertype());
        stmt.setString(7, bean.getSubscrbtype());
        stmt.setString(8, bean.getCurstatus());
        stmt.setTimestamp(9, new Timestamp(System.currentTimeMillis()));
        stmt.setString(10, bean.getOrderchannel());
        stmt.setString(11, bean.getCanceltime());
        stmt.setString(12, bean.getCancelchannel());
        stmt.setString(13, bean.getProvince());
        stmt.setString(14, bean.getCity());
        stmt.setString(15, bean.getProducttype());
        stmt.setString(16, bean.getUseraccount());
        stmt.setString(17, bean.getDirectedtype());
        stmt.setString(18, bean.getNote());
        try {
            int res = stmt.executeUpdate();
            relo = res > 0;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }

    @Override
    public boolean insertDirectedJyLog(DirectedJyLogBean bean) throws Exception {
        String sql = "insert into tl_directed_log(orderid,busiid,cpid,productid,productnewid,usermob," +
                " usertype,subscrbtype,orderchannel,province,city,producttype,useraccount,directedtype," +
                " requesttime,responsetime,requestip,issuccess,resultcode,resultinfo,requestparam,responsecode,responseinfo)" +
                " VALUES( ?, ?, ?, ?, ?, ?" +
                " , ?, ?, ?, ?, ?, ?, ?, ?," +
                "  ?, ?, ?, ?, ?, ?, ?, ?, ?)";
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        stmt = con.prepareStatement(sql);
        stmt.setString(1, bean.getOrderid());
        stmt.setString(2, bean.getBusiid());
        stmt.setString(3, bean.getCpid());
        stmt.setString(4, bean.getProductid());
        stmt.setString(5, bean.getProductnewid());
        stmt.setString(6, bean.getUsermob());
        stmt.setString(7, bean.getUsertype());
        stmt.setString(8, bean.getSubscrbtype());
        stmt.setString(9, bean.getOrderchannel());
        stmt.setString(10, bean.getProvince());
        stmt.setString(11, bean.getCity());
        stmt.setString(12, bean.getProducttype());
        stmt.setString(13, bean.getUseraccount());
        stmt.setString(14, bean.getDirectedtype());
        stmt.setString(15, bean.getRequesttime());
        stmt.setString(16, bean.getResponsetime());
        stmt.setString(17, bean.getRequestip());
        stmt.setString(18, bean.getIssuccess());
        stmt.setString(19, bean.getResultcode());
        stmt.setString(20, bean.getResultinfo());
        stmt.setString(21, bean.getRequestparam());
        stmt.setString(22, bean.getResponsecode());
        stmt.setString(23, bean.getResponseinfo());
        try {
            int res = stmt.executeUpdate();
            relo = res > 0;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }

    @Override
    public boolean cancelOrder(DirectedJyOrderBean bean) throws Exception {
        String sql = "UPDATE td_directed_order SET curstatus= ?,cancelchannel= ?,canceltime= ? where usermob= ? and productid = ?";
        //从连接池 中 取得 连接
        Connection con = JDBCUtils.getMySqlConnection();
        PreparedStatement stmt = null;
        ResultSet rs = null;
        boolean relo = false;
        stmt = con.prepareStatement(sql);
        stmt.setString(1, bean.getCurstatus());
        stmt.setString(2, bean.getCancelchannel());
        stmt.setTimestamp(3, new Timestamp(System.currentTimeMillis()));
        stmt.setString(4, bean.getUsermob());
        stmt.setString(5, bean.getProductid());
        try {
            int res = stmt.executeUpdate();
            relo = res > 0;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.close(rs);
            JDBCUtils.close(stmt);
            JDBCUtils.close(con);
        }
        return relo;
    }
}
