package com.chinacreator.order.send.imp;

import com.chinacreator.order.send.HttpSendService;
import net.sf.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;

public class HttpSendServicImp implements HttpSendService {
    private static final Logger LOGGER = LoggerFactory.getLogger(HttpSendServicImp.class);

    public HttpSendServicImp() {
        super();
    }

    /**
     * 解读流    用于与客户同步订购关系
     *
     * @param inStream
     * @return
     * @throws Exception
     */
    public static String readInputStream(InputStream inStream) throws Exception {
        ByteArrayOutputStream outStream = null;
        try {
            outStream = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int len = 0;
            while ((len = inStream.read(buffer)) != -1) {
                outStream.write(buffer, 0, len);
            }
            byte[] data = outStream.toByteArray();
            return new String(data, StandardCharsets.UTF_8);
        } finally {
            if (outStream != null)
                outStream.close();
            if (inStream != null)
                inStream.close();
        }
    }

    /**
     * 以get方式获取链接的输出  用于与客户同步订购关系
     *
     * @param href
     * @return
     * @throws Exception
     */
    public static String get(String href) throws Exception {
        URL url = new URL(href);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod("GET");
        conn.addRequestProperty("content-type", "text/plain");
        conn.setUseCaches(false);
        conn.setConnectTimeout(30000);
        conn.setDoOutput(false);
        return readInputStream(conn.getInputStream());
    }

    /**
     * 以post方式获取链接的输出  用于与客户同步订购关系
     *
     * @param href
     * @return
     * @throws Exception
     */
    public static String post(String href, String data) throws Exception {
        if (data == null || data.length() == 0) {
            return get(href);
        }

        HttpURLConnection conn = null;
        try {
            URL url = new URL(href);
            conn = (HttpURLConnection) url.openConnection();
            if (conn == null) {
                throw new IllegalArgumentException("The conn cannot be null");
            }
            conn.setRequestMethod("POST");
            conn.addRequestProperty("content-type", "text/plain");
            conn.setDoOutput(true);
            conn.setUseCaches(false);
            OutputStream os = conn.getOutputStream();
            os.write(data.getBytes(StandardCharsets.UTF_8));
            os.flush();
        } finally {
            if (conn != null) {
                conn.getOutputStream().close();
            }
        }
        
        return readInputStream(conn.getInputStream());
    }

    @Override
    public com.alibaba.fastjson.JSONObject sendPost(String requestUrl, String jsonString) {

        JSONObject jsonObject = sendPost(requestUrl, JSONObject.fromObject(jsonString));

        return com.alibaba.fastjson.JSONObject.parseObject(jsonObject.toString());
    }

    /**
     * POST发送接口
     */
    @Override
    public JSONObject sendPost(String requestUrl, JSONObject obj) {
        JSONObject jsonObject = new JSONObject();
        StringBuffer buffer = new StringBuffer();
        InputStream input = null;
        OutputStream output = null;
        try {
            URL url = new URL(requestUrl);
            HttpURLConnection httpUrlConn = (HttpURLConnection) url.openConnection();
            //httpUrlConn.setChunkedStreamingMode(0);//设置块大小 一般上传使用
            //设置连接超时
            httpUrlConn.setConnectTimeout(30000);
            httpUrlConn.setReadTimeout(30000);
            //允许输入输出
            httpUrlConn.setDoOutput(true);
            httpUrlConn.setDoInput(true);
            //不能缓存
            httpUrlConn.setUseCaches(false);
            //设置请求头
            httpUrlConn.setRequestProperty("Connection", "Keep-Alive");
            httpUrlConn.setRequestMethod("POST");
            byte[] data = (obj.toString()).getBytes();
            httpUrlConn.setRequestProperty("Content-Length", String.valueOf(data.length));
            httpUrlConn.setRequestProperty("Content-Type", "application/json;charset=UTF-8");
            httpUrlConn.connect();

            //输出流包含要发送的数据,要注意数据格式编码
            output = httpUrlConn.getOutputStream();
            output.write((obj.toString()).getBytes());

            output.flush();
            output.close();
            output = null;

            //服务器返回东西了，先对响应码判断
            if (httpUrlConn.getResponseCode() == 200) {
                input = httpUrlConn.getInputStream();
                InputStreamReader inputStreamReader = new InputStreamReader(input, StandardCharsets.UTF_8);
                BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
                String str = null;

                while ((str = bufferedReader.readLine()) != null) {
                    buffer.append(str);
                }
                bufferedReader.close();
                inputStreamReader.close();
                input.close();
                input = null;
            }
            httpUrlConn.disconnect();
            if (!buffer.equals(" ")) {
                jsonObject = JSONObject.fromObject(buffer.toString());
            } else {
                jsonObject.put("resultCode", "8001");
                jsonObject.put("errorInfo", "业务调用失败！");
            }
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.error("业务调用失败！  [" + obj.toString() + "], {}", e);
            jsonObject.put("resultCode", "8001");
            jsonObject.put("errorInfo", "业务调用失败，处理异常！");
        } finally {
            try {
                if (output != null) {
                    output.close();
                }
                if (input != null) {
                    input.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
                LOGGER.error("流关闭失败", e);
            }
        }
        return jsonObject;
    }

//	@Override
//	public JSONObject sendPost(String requestUrl, String obj) {
//		logger.info("sendUrl:"+requestUrl+"/n"+"sendData:"+obj);
//		JSONObject jsonObject = new JSONObject();
//		StringBuffer buffer = new StringBuffer();
//		InputStream inputStream = null;
//		try {
//			URL url = new URL(requestUrl);
//			HttpURLConnection httpUrlConn = (HttpURLConnection) url
//					.openConnection();
//			httpUrlConn.setChunkedStreamingMode(0);
//			httpUrlConn.setDoOutput(true);
//			httpUrlConn.setDoInput(true);
//			httpUrlConn.setUseCaches(false);
//			httpUrlConn.setRequestProperty("Connection", "Keep-Alive");
//			httpUrlConn.setRequestMethod("POST");
//			byte[] data = (obj.toString()).getBytes();
//			httpUrlConn.setRequestProperty("Content-Length",
//					String.valueOf(data.length));
//			httpUrlConn.setRequestProperty("contentType", "application/json");
//			httpUrlConn.setRequestProperty("Charset", "UTF-8");
//			httpUrlConn.connect();
//			OutputStream out = httpUrlConn.getOutputStream();
//			out.write(obj.getBytes());
//			out.flush();
//			out.close();
//			logger.info("sendUrl:"+requestUrl+"/n"+"结果编码："+httpUrlConn.getResponseCode());
//			if (httpUrlConn.getResponseCode() == 200) {
//				inputStream = httpUrlConn.getInputStream();
//				InputStreamReader inputStreamReader = new InputStreamReader(
//						inputStream, "UTF-8");
//				BufferedReader bufferedReader = new BufferedReader(
//						inputStreamReader);
//				String str = null;
//				while ((str = bufferedReader.readLine()) != null) {
//					buffer.append(str);
//				}
//				bufferedReader.close();
//				inputStreamReader.close();
//				inputStream.close();
//				inputStream = null;
//			}
//			httpUrlConn.disconnect();
//			jsonObject = JSONObject.fromObject(buffer.toString());
//		}catch (Exception e) {
//			e.printStackTrace();
//		} finally {
//			try {
//				if (inputStream != null) {
//					inputStream.close();
//				}
//			} catch (IOException e) {
//				e.printStackTrace();
//				logger.error("流关闭失败", e);
//			}
//		}
//		return jsonObject;
//	}

}
