package com.chinacreator.order.util;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.sql.DataSource;
import java.sql.*;
import java.util.ArrayList;

public class JDBCUtils {

    private static ComboPooledDataSource dataSource;

    static {
        dataSource = new ComboPooledDataSource();

    }

    // 取得链接
    public static Connection getMySqlConnection() throws SQLException {
        return dataSource.getConnection();
    }

    //获取池
    public static DataSource getDataSource() {
        return dataSource;
    }

    // 关闭链接
    public static void close(Connection conn) throws SQLException {
        Logger logger = LoggerFactory.getLogger(JDBCUtils.class);
        if (conn != null) {
            try {
                // logger.info("关闭Connection(hashCode):" + conn.hashCode());
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
                logger.error("关闭Connection失败:" + conn.hashCode(), e);
                throw e;
            }
        }
    }

    public static void close(PreparedStatement pstate) throws SQLException {
        if (pstate != null) {
            pstate.close();
        }
    }

    public static void close(ResultSet rs) throws SQLException {
        if (rs != null) {
            rs.close();
        }
    }

    /**
     * 包装sql
     *
     * @param stmt
     * @param para
     * @return
     * @throws SQLException
     */
    public static PreparedStatement sqlDecorate(PreparedStatement stmt,
                                                ArrayList<Object> para) throws SQLException {
        for (int i = 0; i < para.size(); i++) {
            JDBCUtils.injectStmt(stmt, i + 1, para.get(i));
        }
        return stmt;
    }

    /**
     * 将参数添加至PreparedStatement对象
     *
     * @param stmt
     * @param i
     * @param para
     * @return
     * @throws SQLException
     */
    private static PreparedStatement injectStmt(PreparedStatement stmt, int i, Object para) throws SQLException {
        if (para instanceof String) {
            stmt.setString(i, para.toString());
        }
        if (para instanceof Integer) {
            stmt.setInt(i, Integer.valueOf(para.toString()));
        }
        if (para instanceof Date) {
            stmt.setDate(i, (Date) para);
        }
        return stmt;
    }


}
